@@ -1,47 +1,7 @@
+from sympy.external import import_module
 from spb.defaults import cfg
 from spb.backends.base_backend import Plot
-from bokeh.plotting import figure, show
-from bokeh.io import output_notebook
-import bokeh.palettes as bp
-from bokeh.models import (
-    LinearColorMapper,
-    ColumnDataSource,
-    MultiLine,
-    ColorBar,
-    Segment,
-)
-from bokeh.models.tickers import FixedTicker
-from bokeh.io import export_png, export_svg
-from bokeh.events import PanEnd
-import itertools
-import colorcet as cc
 import os
-import numpy as np
-from mergedeep import merge
-import matplotlib.pyplot as plt
-import matplotlib.cm as cm
-from spb.backends.utils import convert_colormap
-
-
-def get_contour_data(X, Y, Z, get_source=True):
-    """Uses Matplotlib contours to create a data source to plot contour levels.
-
-    Credit to: https://stackoverflow.com/a/37633519/2329968
-    """
-    cs = plt.contour(X, Y, Z, [0.0])
-    xs = []
-    ys = []
-    for isolevel in cs.collections:
-        for path in isolevel.get_paths():
-            v = path.vertices
-            x = v[:, 0]
-            y = v[:, 1]
-            xs.append(x.tolist())
-            ys.append(y.tolist())
-    data = {"xs": xs, "ys": ys}
-    if not get_source:
-        return data
-    return ColumnDataSource(data=data)
 
 
 def compute_streamlines(x, y, u, v, density=1.0):
@@ -53,8 +13,7 @@ def compute_streamlines(x, y, u, v, density=1.0):
 
     Credit: https://docs.bokeh.org/en/latest/docs/gallery/quiver.html
     """
-
-    # TODO: is it possible to further optimize this function?
+    np = import_module('numpy')
 
     ## Set up some constants - size of the grid used.
     NGX = len(x)
@@ -224,6 +183,10 @@ class BokehBackend(Plot):
     Parameters
     ==========
 
+    aspect : str
+        Set the aspect ratio of a 2D plot. Default to ``None``. Set it to
+        ``"equal"`` to sets equal spacing on the axis.
+
     contour_kw : dict, optional
         A dictionary with keyword arguments to customize the contour.
         This might be useful to change the `palette` to a custom value.
@@ -231,7 +194,7 @@ class BokehBackend(Plot):
     line_kw : dict, optional
         A dictionary of keywords/values which is passed to Plotly's scatter
         functions to customize the appearance. Default to:
-        ``line_kw = dict(line_width = 2)``
+        ``line_kw = dict(line_width = 2)``.
         Refer to [#fn1]_ to learn more about customization.
 
     quiver_kw : dict, optional
@@ -254,7 +217,11 @@ class BokehBackend(Plot):
     theme : str, optional
         Set the theme. Default to ``"dark_minimal"``. Find more Bokeh themes
         at [#fn2]_ .
-        
+
+    update_event : bool, optional
+        If True, the backend will update the data series over the visibile
+        range whenever a pan-event is triggered. Default to True.
+
 
     References
     ==========
@@ -270,40 +237,56 @@ class BokehBackend(Plot):
 
     _library = "bokeh"
 
-    colorloop = bp.Category10[10]
-
-    # to be used in parametric plots
-    colormaps = [cc.bmy, "aggrnyl", cc.kbc, cc.bjy, "plotly3"]
-
-    # to be used in complex-parametric plots
-    cyclic_colormaps = [cm.hsv, cm.twilight, cc.cyclic_mygbm_30_95_c78_s25]
+    colorloop = []
+    colormaps = []
+    cyclic_colormaps = []
 
     def __new__(cls, *args, **kwargs):
-        # Since Plot has its __new__ method, this will prevent infinite
-        # recursion
         return object.__new__(cls)
 
     def __init__(self, *args, **kwargs):
+        self.bokeh = import_module(
+            'bokeh',
+            import_kwargs={'fromlist': ['models', 'events', 'plotting', 'io', 'palettes', 'embed', 'resources']},
+            min_module_version='2.3.0')
+        bp = self.bokeh.palettes
+        cc = import_module(
+            'colorcet',
+            min_module_version='3.0.0')
+        matplotlib = import_module(
+            'matplotlib',
+            import_kwargs={'fromlist': ['pyplot', 'cm']},
+            min_module_version='1.1.0',
+            catch=(RuntimeError,))
+        cm = matplotlib.cm
+
+        self.colorloop = bp.Category10[10]
+        self.colormaps = [cc.bmy, "aggrnyl", cc.kbc, cc.bjy, "plotly3"]
+        self.cyclic_colormaps = [cm.hsv, cm.twilight, cc.cyclic_mygbm_30_95_c78_s25]
+
+        self._init_cyclers()
         super().__init__(*args, **kwargs)
 
+        self._theme = kwargs.get("theme", cfg["bokeh"]["theme"])
+        self._update_event = kwargs.get("update_event",
+            cfg["bokeh"]["update_event"])
+
         self._run_in_notebook = False
         if self._get_mode() == 0:
             self._run_in_notebook = True
-            output_notebook(hide_banner=True)
+            self.bokeh.io.output_notebook(hide_banner=True)
 
-        # add colors if needed
-        if (len([s for s in self._series if s.is_2Dline]) > 10) and (
-            self.colorloop == bp.Category10[10]
-        ):
+        if ((len([s for s in self._series if s.is_2Dline]) > 10) and
+            (not type(self).colorloop) and
+            not ("process_piecewise" in kwargs.keys())):
+            # add colors if needed
             self.colorloop = bp.Category20[20]
 
         self._handles = dict()
-        self._init_cyclers()
 
+        # empty plots (len(series)==0) should only have x, y tooltips
         TOOLTIPS = [("x", "$x"), ("y", "$y")]
         if len(self.series) > 0:
-            # empty plots (with len(series)==0) should only have x, y tooltips
-
             if all([s.is_parametric for s in self.series]):
                 # with parametric plots, also visualize the parameter
                 TOOLTIPS += [("u", "@us")]
@@ -318,7 +301,7 @@ class BokehBackend(Plot):
             # for complex domain coloring
             sizing_mode = None
 
-        self._fig = figure(
+        self._fig = self.bokeh.plotting.figure(
             title=self.title,
             x_axis_label=self.xlabel if self.xlabel else "x",
             y_axis_label=self.ylabel if self.ylabel else "y",
@@ -333,19 +316,51 @@ class BokehBackend(Plot):
             tooltips=TOOLTIPS,
             match_aspect=True if self.aspect == "equal" else False,
         )
-        self._fig.axis.visible = self.grid
         self._fig.grid.visible = self.grid
-        self._fig.on_event(PanEnd, self._pan_update)
+        if cfg["bokeh"]["show_minor_grid"]:
+            self._fig.grid.minor_grid_line_alpha = cfg["bokeh"]["minor_grid_line_alpha"]
+            self._fig.grid.minor_grid_line_color = self._fig.grid.grid_line_color[0]
+            self._fig.grid.minor_grid_line_dash = cfg["bokeh"]["minor_grid_line_dash"]
+        self._fig.on_event(self.bokeh.events.PanEnd, self._pan_update)
+
+    @property
+    def fig(self):
+        """Returns the figure."""
+        if len(self.series) != len(self._fig.renderers):
+            # if the backend was created without showing it
+            self.process_series()
+        return self._fig
+
+    def process_series(self):
+        """ Loop over data series, generates numerical data and add it to the
+        figure.
+        """
         self._process_series(self._series)
 
+    def _set_piecewise_color(self, s, color):
+        """Set the color to the given series"""
+        s.rendering_kw["color"] = color
+        if s.is_point and (not s.is_filled):
+            s.rendering_kw["fill_color"] = "white"
+
+    @staticmethod
+    def _do_sum_kwargs(p1, p2):
+        kw = p1._copy_kwargs()
+        kw["theme"] = p1._theme
+        return kw
+
     def _process_series(self, series):
+        np = import_module('numpy')
+        merge = self.merge
         self._init_cyclers()
-        # clear figure. need to clear both the renderers as well as the
+        # clear figure. Must clear both the renderers as well as the
         # colorbars which are added to the right side.
         self._fig.renderers = []
         self._fig.right = []
 
         for i, s in enumerate(series):
+            kw = None
+
             if s.is_2Dline:
                 if s.is_parametric and self._use_cm:
                     x, y, param = s.get_data()
@@ -354,9 +369,8 @@ class BokehBackend(Plot):
                         if self._use_cyclic_cm(param, s.is_complex)
                         else next(self._cm)
                     )
-                    ds, line, cb = self._create_gradient_line(
-                        x, y, param, colormap, s.label
-                    )
+                    ds, line, cb, kw = self._create_gradient_line(
+                        x, y, param, colormap, s.label, s.rendering_kw)
                     self._fig.add_glyph(ds, line)
                     if self.legend:
                         self._handles[i] = cb
@@ -367,25 +381,19 @@ class BokehBackend(Plot):
                         source = {"xs": x, "ys": y, "us": param}
                     else:
                         x, y = s.get_data()
-                        x, y, modified = self._detect_poles(x, y)
-                        if modified:
-                            self._fig.y_range.start = self.ylim[0]
-                            self._fig.y_range.end = self.ylim[1]
                         source = {"xs": x, "ys": y}
 
-                    lkw = dict(line_width=2, legend_label=s.label, color=next(self._cl))
-                    line_kw = self._kwargs.get("line_kw", dict())
+                    lkw = dict(line_width=2, legend_label=s.label,
+                        color=next(self._cl))
                     if not s.is_point:
-                        self._fig.line(
-                            "xs", "ys", source=source, **merge({}, lkw, line_kw)
-                        )
+                        kw = merge({}, lkw, s.rendering_kw)
+                        self._fig.line("xs", "ys", source=source, **kw)
                     else:
-                        self._fig.dot(
-                            "xs",
-                            "ys",
-                            source=source,
-                            **merge({"size": 20}, lkw, line_kw)
-                        )
+                        lkw["size"] = 8
+                        if not s.is_filled:
+                            lkw["fill_color"] = "white"
+                        kw = merge({}, lkw, s.rendering_kw)
+                        self._fig.circle("xs", "ys", source=source, **kw)
 
             elif s.is_contour and (not s.is_complex):
                 x, y, z = s.get_data()
@@ -393,8 +401,9 @@ class BokehBackend(Plot):
                 minx, miny, minz = min(x), min(y), min(zz)
                 maxx, maxy, maxz = max(x), max(y), max(zz)
 
-                contour_kw = self._kwargs.get("contour_kw", dict()).copy()
-                cm = contour_kw.pop("palette", next(self._cm))
+                cm = next(self._cm)
+                ckw = dict(palette=cm)
+                kw = merge({}, ckw, s.rendering_kw)
 
                 self._fig.image(
                     image=[z],
@@ -402,105 +411,51 @@ class BokehBackend(Plot):
                     y=miny,
                     dw=abs(maxx - minx),
                     dh=abs(maxy - miny),
-                    palette=cm,
+                    **kw
                 )
 
-                colormapper = LinearColorMapper(palette=cm, low=minz, high=maxz)
+                colormapper = self.bokeh.models.LinearColorMapper(
+                    palette=cm, low=minz, high=maxz)
                 cbkw = dict(width=8, title=s.label)
-                colorbar = ColorBar(color_mapper=colormapper, **cbkw)
+                colorbar = self.bokeh.models.ColorBar(
+                    color_mapper=colormapper, **cbkw)
                 self._fig.add_layout(colorbar, "right")
                 self._handles[i] = colorbar
 
-            elif s.is_implicit:
-                points = s.get_data()
-                # TODO: add color to the legend
-                if len(points) == 2:
-                    # interval math plotting
-                    x, y, pixels = self._get_pixels(s, points[0])
-                    x, y = x.flatten(), y.flatten()
-                    cm = ["#00000000", next(self._cl)]
-                    self._fig.image(
-                        image=[pixels],
-                        x=min(x),
-                        y=min(y),
-                        dw=abs(max(x) - min(x)),
-                        dh=abs(max(y) - min(y)),
-                        palette=cm,
-                        legend_label=s.label,
-                    )
-                else:
-                    x, y, z, ones, plot_type = points
-                    if plot_type == "contour":
-                        source = get_contour_data(x, y, z)
-                        lkw = dict(
-                            line_color=next(self._cl),
-                            line_width=2,
-                            source=source,
-                            legend_label=s.label,
-                        )
-                        line_kw = self._kwargs.get("line_kw", dict())
-                        self._fig.multi_line("xs", "ys", **merge({}, lkw, line_kw))
-                    else:
-                        cm = ["#00000000", next(self._cl)]
-                        self._fig.image(
-                            image=[ones],
-                            x=min(x),
-                            y=min(y),
-                            dw=abs(max(x) - min(x)),
-                            dh=abs(max(y) - min(y)),
-                            palette=cm,
-                            legend_label=s.label,
-                        )
-
             elif s.is_2Dvector:
                 if s.is_streamlines:
                     x, y, u, v = s.get_data()
                     sqk = dict(color=next(self._cl), line_width=2, line_alpha=0.8)
-                    stream_kw = self._kwargs.get("stream_kw", dict())
+                    stream_kw = s.rendering_kw.copy()
                     density = stream_kw.pop("density", 2)
+                    kw = merge({}, sqk, stream_kw)
                     xs, ys = compute_streamlines(
-                        x[0, :], y[:, 0], u, v, density=density
-                    )
-                    self._fig.multi_line(xs, ys, **merge({}, sqk, stream_kw))
+                        x[0, :], y[:, 0], u, v, density=density)
+                    self._fig.multi_line(xs, ys, **kw)
                 else:
                     x, y, u, v = s.get_data()
-                    quiver_kw = self._kwargs.get("quiver_kw", dict())
-                    data, quiver_kw = self._get_quivers_data(x, y, u, v, **quiver_kw)
+                    data, quiver_kw = self._get_quivers_data(x, y, u, v,
+                        **s.rendering_kw.copy())
                     mag = data["magnitude"]
 
-                    color_mapper = LinearColorMapper(
-                        palette=next(self._cm), low=min(mag), high=max(mag)
-                    )
+                    color_mapper = self.bokeh.models.LinearColorMapper(
+                        palette=next(self._cm), low=min(mag), high=max(mag))
                     # don't use color map if a scalar field is visible or if
                     # use_cm=False
-                    solid = (
-                        True
-                        if ("scalar" not in self._kwargs.keys())
-                        else (
-                            False
-                            if (
-                                (not self._kwargs["scalar"])
-                                or (self._kwargs["scalar"] is None)
-                            )
-                            else True
-                        )
-                    )
                     line_color = (
                         {"field": "magnitude", "transform": color_mapper}
-                        if ((not solid) and self._use_cm)
+                        if ((not s.use_quiver_solid_color) and self._use_cm)
                         else next(self._cl)
                     )
-                    source = ColumnDataSource(data=data)
-                    # default quivers options
+                    source = self.bokeh.models.ColumnDataSource(data=data)
                     qkw = dict(line_color=line_color, line_width=1, name=s.label)
-                    glyph = Segment(
-                        x0="x0", y0="y0", x1="x1", y1="y1", **merge({}, qkw, quiver_kw)
-                    )
+                    kw = merge({}, qkw, quiver_kw)
+                    glyph = self.bokeh.models.Segment(
+                        x0="x0", y0="y0", x1="x1", y1="y1", **kw)
                     self._fig.add_glyph(source, glyph)
                     if isinstance(line_color, dict):
-                        colorbar = ColorBar(
-                            color_mapper=color_mapper, width=8, title=s.label
-                        )
+                        colorbar = self.bokeh.models.ColorBar(
+                            color_mapper=color_mapper, width=8, title=s.label)
                         self._fig.add_layout(colorbar, "right")
                         self._handles[i] = colorbar
 
@@ -508,7 +463,7 @@ class BokehBackend(Plot):
                 x, y, mag, angle, img, colors = s.get_data()
                 img = self._get_img(img)
 
-                source = ColumnDataSource(
+                source = self.bokeh.models.ColumnDataSource(
                     {
                         "image": [img],
                         "abs": [mag],
@@ -526,25 +481,24 @@ class BokehBackend(Plot):
 
                 if colors is not None:
                     # chroma/phase-colorbar
-                    cm1 = LinearColorMapper(
-                        palette=[tuple(c) for c in colors], low=-np.pi, high=np.pi
-                    )
+                    cm1 = self.bokeh.models.LinearColorMapper(
+                        palette=[tuple(c) for c in colors],
+                        low=-np.pi, high=np.pi)
                     ticks = [-np.pi, -np.pi / 2, 0, np.pi / 2, np.pi]
                     labels = ["-π", "-π / 2", "0", "π / 2", "π"]
-                    colorbar1 = ColorBar(
+                    colorbar1 = self.bokeh.models.ColorBar(
                         color_mapper=cm1,
                         title="Argument",
-                        ticker=FixedTicker(ticks=ticks),
-                        major_label_overrides={k: v for k, v in zip(ticks, labels)},
-                    )
+                        ticker=self.bokeh.models.tickers.FixedTicker(ticks=ticks),
+                        major_label_overrides={k: v for k, v in zip(ticks, labels)})
                     self._fig.add_layout(colorbar1, "right")
 
             elif s.is_geometry:
                 x, y = s.get_data()
                 color = next(self._cl)
                 pkw = dict(alpha=0.5, line_width=2, line_color=color, fill_color=color)
-                patch_kw = self._kwargs.get("patch_kw", dict())
-                self._fig.patch(x, y, **merge({}, pkw, patch_kw))
+                kw = merge({}, pkw, s.rendering_kw)
+                self._fig.patch(x, y, **kw)
 
             else:
                 raise NotImplementedError(
@@ -565,7 +519,6 @@ class BokehBackend(Plot):
                 s.start = self._fig.x_range.start
                 s.end = self._fig.x_range.end
                 x, y = s.get_data()
-                x, y, _ = self._detect_poles(x, y)
                 source = {"xs": x, "ys": y}
                 rend[i].data_source.data.update(source)
             elif s.is_complex and s.is_2Dline and s.is_parametric and self._use_cm:
@@ -580,6 +533,7 @@ class BokehBackend(Plot):
                     cb.color_mapper.update(low=min(us), high=max(us))
 
     def _get_img(self, img):
+        np = import_module('numpy')
         new_img = np.zeros(img.shape[:2], dtype=np.uint32)
         pixel = new_img.view(dtype=np.uint8).reshape((*img.shape[:2], 4))
         for i in range(img.shape[1]):
@@ -591,39 +545,40 @@ class BokehBackend(Plot):
         # MultiLine works with line segments, not with line points! :|
         xs = [x[i - 1 : i + 1] for i in range(1, len(x))]
         ys = [y[i - 1 : i + 1] for i in range(1, len(y))]
-        # TODO: let n be the number of points. Then, the number of segments will
-        # be (n - 1). Therefore, we remove one parameter. If n is sufficiently
-        # high, there shouldn't be any noticeable problem in the visualization.
+        # let n be the number of points. Then, the number of segments
+        # will be (n - 1). Therefore, we remove one parameter. If n is
+        # sufficiently high, there shouldn't be any noticeable problem in
+        # the visualization.
         us = u[:-1]
         return xs, ys, us
 
-    def _create_gradient_line(self, x, y, u, colormap, name):
+    def _create_gradient_line(self, x, y, u, colormap, name, line_kw):
+        merge = self.merge
         xs, ys, us = self._get_segments(x, y, u)
-        color_mapper = LinearColorMapper(palette=colormap, low=min(us), high=max(us))
-        data_source = ColumnDataSource(dict(xs=xs, ys=ys, us=us))
+        color_mapper = self.bokeh.models.LinearColorMapper(
+            palette=colormap, low=min(us), high=max(us))
+        data_source = self.bokeh.models.ColumnDataSource(
+            dict(xs=xs, ys=ys, us=us))
 
         lkw = dict(
             line_width=2,
             name=name,
             line_color={"field": "us", "transform": color_mapper},
         )
-        line_kw = self._kwargs.get("line_kw", dict())
-        glyph = MultiLine(xs="xs", ys="ys", **merge({}, lkw, line_kw))
-        # default options
-        cbkw = dict(width=8)
-        # user defined options
-        colorbar_kw = self._kwargs.get("colorbar_kw", dict())
-        colorbar = ColorBar(
-            color_mapper=color_mapper, title=name, **merge({}, cbkw, colorbar_kw)
-        )
-        return data_source, glyph, colorbar
+        kw = merge({}, lkw, line_kw)
+        glyph = self.bokeh.models.MultiLine(xs="xs", ys="ys", **kw)
+        colorbar = self.bokeh.models.ColorBar(
+            color_mapper=color_mapper, title=name, width=8)
+        return data_source, glyph, colorbar, kw
 
     def _update_interactive(self, params):
         rend = self.fig.renderers
+        if len(rend) != len(self.series):
+            self._process_series(self.series)
 
         for i, s in enumerate(self.series):
             if s.is_interactive:
-                self.series[i].update_data(params)
+                self.series[i].params = params
 
                 if s.is_2Dline and s.is_parametric and self._use_cm:
                     x, y, param = self.series[i].get_data()
@@ -639,7 +594,6 @@ class BokehBackend(Plot):
                         source = {"xs": x, "ys": y, "us": param}
                     else:
                         x, y = self.series[i].get_data()
-                        x, y, _ = self._detect_poles(x, y)
                         source = {"xs": x, "ys": y}
                     rend[i].data_source.data.update(source)
 
@@ -648,46 +602,29 @@ class BokehBackend(Plot):
                     cb = self._handles[i]
                     rend[i].data_source.data.update({"image": [z]})
                     zz = z.flatten()
-                    # TODO: as of Bokeh 2.3.2, the following line going to update
-                    # the values of the color mapper, but the redraw is not
-                    # applied, hence there is an error in the visualization.
-                    # Keep track of the following issue:
+                    # TODO: as of Bokeh 2.3.2, the following line is going to
+                    # update the values of the color mapper, but the redraw
+                    # is not applied, hence there is an error in the
+                    # visualization. Keep track of the following issue:
                     # https://github.com/bokeh/bokeh/issues/11116
                     cb.color_mapper.update(low=min(zz), high=max(zz))
 
-                elif s.is_implicit:
-                    points = s.get_data()
-                    if len(points) == 2:
-                        raise NotImplementedError
-                    else:
-                        x, y, z, ones, plot_type = points
-                        if plot_type == "contour":
-                            data = get_contour_data(x, y, z, False)
-                            # TODO: for some unkown reason, this is not updating!
-                            rend[i].data_source.data.update(data)
-                        else:
-                            source = {"image": [ones]}
-                            rend[i].data_source.data.update(source)
-
                 elif s.is_2Dvector:
                     x, y, u, v = s.get_data()
                     if s.is_streamlines:
-                        sqk = dict(color=next(self._cl), line_width=2, line_alpha=0.8)
-                        stream_kw = self._kwargs.get("stream_kw", dict())
-                        density = stream_kw.pop("density", 2)
+                        density = s.rendering_kw.copy().pop("density", 2)
                         xs, ys = compute_streamlines(
                             x[0, :], y[:, 0], u, v, density=density
                         )
                         rend[i].data_source.data.update({"xs": xs, "ys": ys})
                     else:
-                        quiver_kw = self._kwargs.get("quiver_kw", dict())
+                        quiver_kw = s.rendering_kw.copy()
                         data, quiver_kw = self._get_quivers_data(
                             x, y, u, v, **quiver_kw
                         )
                         mag = data["magnitude"]
-                        color_mapper = LinearColorMapper(
-                            palette=next(self._cm), low=min(mag), high=max(mag)
-                        )
+                        color_mapper = self.bokeh.models.LinearColorMapper(
+                            palette=next(self._cm), low=min(mag), high=max(mag))
                         line_color = quiver_kw.get(
                             "line_color",
                             {"field": "magnitude", "transform": color_mapper},
@@ -720,9 +657,9 @@ class BokehBackend(Plot):
 
         Notes
         =====
-        
-        1. In order to export static pictures, the user also need to install the
-           packages listed in [#fn5]_.
+
+        1. In order to export static pictures, the user also need to install
+           the packages listed in [#fn5]_.
         2. When exporting a fully portable html file, by default the necessary
            Javascript libraries will be loaded with a CDN. This creates the
            smallest file size possible, but it requires an internet connection
@@ -735,48 +672,49 @@ class BokehBackend(Plot):
         .. [#fn5] https://docs.bokeh.org/en/latest/docs/reference/io.html#module-bokeh.io.export
 
         """
-        # TODO: an error get raised if I uncomment the following line
-        # self._process_series(self._series)
+        merge = self.merge
+
         ext = os.path.splitext(path)[1]
         if ext.lower() in [".html", ".html"]:
-            from bokeh.resources import CDN
-            from bokeh.embed import file_html
+            CDN = self.bokeh.resources.CDN
+            file_html = self.bokeh.embed.file_html
             skw = dict(resources=CDN, title="Bokeh Plot")
             html = file_html(self.fig, **merge(skw, kwargs))
             with open(path, 'w') as f:
                 f.write(html)
         elif ext == ".svg":
             self._fig.output_backend = "svg"
-            export_svg(self.fig, filename=path)
+            self.bokeh.io.export_svg(self.fig, filename=path)
         else:
             if ext == "":
                 path += ".png"
             self._fig.output_backend = "canvas"
-            export_png(self._fig, filename=path)
+            self.bokeh.io.export_png(self._fig, filename=path)
 
     def _launch_server(self, doc):
         """By launching a server application, we can use Python callbacks
         associated to events.
         """
-        doc.theme = self._kwargs.get("theme", cfg["bokeh"]["theme"])
+        doc.theme = self._theme
         doc.add_root(self._fig)
 
     def show(self):
         """Visualize the plot on the screen."""
-        self._process_series(self._series)
-        if self._run_in_notebook:
-            # TODO: the current way we are launching the server only works within
-            # Jupyter Notebook. Is there another way of launching it so that it can
-            # run from any Python interpreter?
-            show(self._launch_server)
+        if len(self._fig.renderers) != len(self.series):
+            self._process_series(self._series)
+        if self._run_in_notebook and self._update_event:
+            # TODO: the current way we are launching the server only works
+            # within Jupyter Notebook. Is there another way of launching
+            # it so that it can run from any Python interpreter?
+            self.bokeh.plotting.show(self._launch_server)
         else:
             # if the backend it running from a python interpreter, the server
             # wont' work. Hence, launch a static figure, which doesn't listen
             # to events (no pan-auto-update).
-            from bokeh.io import curdoc
+            curdoc = self.bokeh.io.curdoc
 
-            curdoc().theme = self._kwargs.get("theme", cfg["bokeh"]["theme"])
-            show(self._fig)
+            curdoc().theme = self._theme
+            self.bokeh.plotting.show(self._fig)
 
     def _get_quivers_data(self, xs, ys, u, v, **quiver_kw):
         """Compute the segments coordinates to plot quivers.
@@ -810,6 +748,7 @@ class BokehBackend(Plot):
                 A dictionary containing keywords to customize the appearance
                 of Bokeh's Segment glyph
         """
+        np = import_module('numpy')
         scale = quiver_kw.pop("scale", 1.0)
         pivot = quiver_kw.pop("pivot", "mid")
         arrow_heads = quiver_kw.pop("arrow_heads", True)
