@@ -1,22 +1,9 @@
 from spb.defaults import cfg
 from spb.backends.base_backend import Plot
 from spb.backends.utils import compute_streamtubes
-import matplotlib.pyplot as plt
-import matplotlib.cm as cm
-from matplotlib.collections import LineCollection
-from matplotlib.colors import ListedColormap, Normalize
-import mpl_toolkits
-from mpl_toolkits.mplot3d.art3d import Line3DCollection, Path3DCollection
-import numpy as np
-from mergedeep import merge
+from sympy.external import import_module
 import itertools
 
-"""
-TODO:
-    1. Besides the axis on the center of the image, there are also a couple of
-        axis with ticks on the bottom/right sides. Delete them?
-"""
-
 # Global variable
 # Set to False when running tests / doctests so that the plots don't show.
 _show = True
@@ -32,7 +19,7 @@ def unset_show():
 
 def _matplotlib_list(interval_list):
     """
-    Returns lists for matplotlib ``fill`` command from a list of bounding
+    Returns lists for matplotlib `fill` command from a list of bounding
     rectangular intervals
     """
     xlist = []
@@ -48,7 +35,7 @@ def _matplotlib_list(interval_list):
                 [intervaly.start, intervaly.end, intervaly.end, intervaly.start, None]
             )
     else:
-        # XXX Ugly hack. Matplotlib does not accept empty lists for ``fill``
+        # XXX Ugly hack. Matplotlib does not accept empty lists for `fill`
         xlist.extend([None, None, None, None])
         ylist.extend([None, None, None, None])
     return xlist, ylist
@@ -62,13 +49,13 @@ class MatplotlibBackend(Plot):
     ==========
 
     aspect : (float, float) or str, optional
-        Set the aspect ratio of the plot. Possible values:
+        Set the aspect ratio of a 2D plot. Possible values:
 
         * ``'auto'``: Matplotlib will fit the plot in the vibile area.
         * ``"equal"``: sets equal spacing on the axis of a 2D plot.
         * tuple containing 2 float numbers, from which the aspect ratio is
           computed.
-    
+
     axis_center : (float, float) or str or None, optional
         Set the location of the intersection between the horizontal and
         vertical axis in a 2D plot. It can be:
@@ -116,7 +103,7 @@ class MatplotlibBackend(Plot):
     use_cm : boolean, optional
         If True, apply a color map to the mesh/surface or parametric lines.
         If False, solid colors will be used instead. Default to True.
-    
+
 
     References
     ==========
@@ -137,35 +124,62 @@ class MatplotlibBackend(Plot):
 
     _library = "matplotlib"
 
-    colormaps = [
-        cm.viridis, cm.autumn, cm.winter, cm.plasma, cm.jet,
-        cm.gnuplot, cm.brg, cm.coolwarm, cm.cool, cm.summer,
-    ]
-
-    cyclic_colormaps = [cm.twilight, cm.hsv]
+    colormaps = []
+    cyclic_colormaps = []
 
     def __new__(cls, *args, **kwargs):
-        # Since Plot has its __new__ method, this will prevent infinite
-        # recursion
         return object.__new__(cls)
 
     def __init__(self, *args, **kwargs):
-        # set global options like title, axis labels, ...
+        self.matplotlib = import_module(
+            'matplotlib',
+            import_kwargs={'fromlist': ['pyplot', 'cm', 'collections', 'colors']},
+            min_module_version='1.1.0',
+            catch=(RuntimeError,))
+        self.plt = self.matplotlib.pyplot
+        self.cm = cm = self.matplotlib.cm
+        self.LineCollection = self.matplotlib.collections.LineCollection
+        self.ListedColormap = self.matplotlib.colors.ListedColormap
+        self.Normalize = self.matplotlib.colors.Normalize
+
+        # set default colors
+        self.colormaps = [
+            cm.viridis, cm.autumn, cm.winter, cm.plasma, cm.jet,
+            cm.gnuplot, cm.brg, cm.coolwarm, cm.cool, cm.summer]
+        self.cyclic_colormaps = [cm.twilight, cm.hsv]
+        self.colorloop = cm.tab10.colors
+
+        self._init_cyclers()
         super().__init__(*args, **kwargs)
 
-        # add colors if needed
-        if (len([s for s in self._series if s.is_2Dline]) > 10) and (
-            self.colorloop == cm.tab10
-        ):
-            self.colorloop = cm.tab20
+        if ((len([s for s in self._series if s.is_2Dline]) > 10) and
+            (not type(self).colorloop) and
+            not ("process_piecewise" in kwargs.keys())):
+            # add colors if needed
+            self.colorloop = cm.tab20.colors
+
+        # plotgrid() can provide its figure and axes to be populated with
+        # the data from the series.
+        self._plotgrid_fig = kwargs.pop("fig", None)
+        self._plotgrid_ax = kwargs.pop("ax", None)
 
         if self.axis_center is None:
             self.axis_center = cfg["matplotlib"]["axis_center"]
-        # see self._add_handle for more info about the following dictionary
+        self._show_minor_grid = kwargs.get("show_minor_grid", cfg["matplotlib"]["show_minor_grid"])
+
         self._handles = dict()
 
+    def _set_piecewise_color(self, s, color):
+        """Set the color to the given series"""
+        s.rendering_kw["color"] = color
+
+    @staticmethod
+    def _do_sum_kwargs(p1, p2):
+        return p1._copy_kwargs()
+
     def _init_cyclers(self):
         super()._init_cyclers()
+        np = import_module('numpy')
 
         # For flexibily, spb.backends.utils.convert_colormap returns numpy
         # ndarrays whenever plotly/colorcet/k3d color map are given. Here we
@@ -174,35 +188,21 @@ class MatplotlibBackend(Plot):
             cm = []
             for i in range(len(colormaps)):
                 c = next(it)
-                cm.append(c if not isinstance(c, np.ndarray) else ListedColormap(c))
+                cm.append(c if not isinstance(c, np.ndarray) else self.ListedColormap(c))
             return itertools.cycle(cm)
 
         self._cm = process_iterator(self._cm, self.colormaps)
         self._cyccm = process_iterator(self._cyccm, self.cyclic_colormaps)
 
     def _create_figure(self):
-        # the following import is here in order to avoid a circular import error
-        from spb.defaults import cfg
-
-        use_jupyterthemes = cfg["matplotlib"]["use_jupyterthemes"]
-        mpl_jupytertheme = cfg["matplotlib"]["jupytertheme"]
-        if (self._get_mode() == 0) and use_jupyterthemes:
-            # set matplotlib style to match the used Jupyter theme
-            try:
-                from jupyterthemes import jtplot
-
-                jtplot.style(mpl_jupytertheme)
-            except:
-                pass
-
         is_3Dvector = any([s.is_3Dvector for s in self.series])
         aspect = self.aspect
         if aspect != "auto":
             if aspect == "equal" and is_3Dvector:
                 # plot_vector uses an aspect="equal" by default. In that case
                 # we would get:
-                # NotImplementedError: Axes3D currently only supports the aspect
-                # argument 'auto'. You passed in 1.0.
+                # NotImplementedError: Axes3D currently only supports the
+                # aspect argument 'auto'. You passed in 1.0.
                 # This fixes it
                 aspect = "auto"
             elif aspect == "equal":
@@ -213,16 +213,19 @@ class MatplotlibBackend(Plot):
             else:
                 aspect = float(aspect[1]) / aspect[0]
 
-        if self._kwargs.get("fig", None) is not None:
-            # We assume we are generating a PlotGrid object, hence the figure
-            # and the axes are provided by the user.
-            self._fig = self._kwargs.pop("fig", None)
-            self.ax = self._kwargs.pop("ax", None)
+        if self._plotgrid_fig is not None:
+            self._fig = self._plotgrid_fig
+            self.ax = self._plotgrid_ax
         else:
-            self._fig = plt.figure(figsize=self.size)
+            if not self.is_iplot:
+                self._fig = self.plt.figure(figsize=self.size)
+            else:
+                self._fig = self.matplotlib.figure.Figure(figsize=self.size)
+
             is_3D = [s.is_3D for s in self.series]
             if any(is_3D) and (not all(is_3D)):
-                raise ValueError("The matplotlib backend can not mix 2D and 3D.")
+                raise ValueError(
+                    "The matplotlib backend can not mix 2D and 3D.")
 
             kwargs = dict(aspect=aspect)
             if all(is_3D):
@@ -231,8 +234,11 @@ class MatplotlibBackend(Plot):
 
     @property
     def fig(self):
-        """Returns the objects used to render/display the plots"""
-        return self._fig, self.ax
+        """Returns the figure."""
+        if (not hasattr(self, "ax")):
+            # if the backend was created without showing it
+            self.process_series()
+        return self._fig
 
     @staticmethod
     def get_segments(x, y, z=None):
@@ -251,6 +257,7 @@ class MatplotlibBackend(Plot):
             z: list
                 List of z-coordinates for a 3D line.
         """
+        np = import_module('numpy')
         if z is not None:
             dim = 3
             points = (x, y, z)
@@ -263,28 +270,33 @@ class MatplotlibBackend(Plot):
     def _add_colorbar(self, c, label, override=False):
         """Add a colorbar for the specificied collection
 
-        Keyword Aurguments
-        ==================
-            override : boolean
-                For parametric plots the colorbar acts like a legend. Hence,
-                when legend=False we don't display the colorbar. However,
-                for contour plots the colorbar is essential to understand it.
-                Hence, to show it we set override=True.
-                Default to False.
+        Parameters
+        ==========
+
+        c : collection
+
+        label : str
+
+        override : boolean
+            For parametric plots the colorbar acts like a legend. Hence,
+            when legend=False we don't display the colorbar. However,
+            for contour plots the colorbar is essential to understand it.
+            Hence, to show it we set override=True.
+            Default to False.
         """
         # design choice: instead of showing a legend entry (which
         # would require to work with proxy artists and custom
         # classes in order to create a gradient line), just show a
         # colorbar with the name of the expression on the side.
         if (self.legend and self._use_cm) or override:
-            # TODO: colorbar position? used space?
             cb = self._fig.colorbar(c, ax=self.ax)
             cb.set_label(label, rotation=90)
             return True
         return False
 
     def _add_handle(self, i, h, kw=None, *args):
-        """self._handle is a dictionary where:
+        """self._handle is a dictionary which will be used with iplot.
+        In particular:
             key: integer corresponding to the i-th series.
             value: a list of two elements:
                 1. handle of the object created by Matplotlib commands
@@ -292,11 +304,18 @@ class MatplotlibBackend(Plot):
                     Some object can't be updated, hence we need to reconstruct
                     it from scratch at every update.
                 3. anything else needed to reconstruct the object.
-        This dictionary will be used with iplot
         """
         self._handles[i] = [h if not isinstance(h, (list, tuple)) else h[0], kw, *args]
 
     def _process_series(self, series):
+        np = import_module('numpy')
+        mpl_toolkits = import_module(
+            'mpl_toolkits', # noqa
+            import_kwargs={'fromlist': ['mplot3d']},
+            catch=(RuntimeError,))
+        Line3DCollection = mpl_toolkits.mplot3d.art3d.Line3DCollection
+        merge = self.merge
+
         # XXX Workaround for matplotlib issue
         # https://github.com/matplotlib/matplotlib/issues/17130
         xlims, ylims, zlims = [], [], []
@@ -305,20 +324,20 @@ class MatplotlibBackend(Plot):
         self._init_cyclers()
 
         for i, s in enumerate(series):
+            kw = None
+
             if s.is_2Dline:
-                line_kw = self._kwargs.get("line_kw", dict())
                 if s.is_parametric and self._use_cm:
                     x, y, param = s.get_data()
-                    x, y, _ = self._detect_poles(x, y)
                     colormap = (
                         next(self._cyccm)
                         if self._use_cyclic_cm(param, s.is_complex)
                         else next(self._cm)
                     )
                     lkw = dict(array=param, cmap=colormap)
-                    kw = merge({}, lkw, line_kw)
+                    kw = merge({}, lkw, s.rendering_kw)
                     segments = self.get_segments(x, y)
-                    c = LineCollection(segments, **kw)
+                    c = self.LineCollection(segments, **kw)
                     self.ax.add_collection(c)
                     is_cb_added = self._add_colorbar(c, s.label)
                     self._add_handle(i, c, kw, is_cb_added, self._fig.axes[-1])
@@ -327,83 +346,49 @@ class MatplotlibBackend(Plot):
                         x, y, param = s.get_data()
                     else:
                         x, y = s.get_data()
-                    x, y, _ = self._detect_poles(x, y)
                     lkw = dict(label=s.label, color=next(self._cl))
                     if s.is_point:
                         lkw["marker"] = "o"
                         lkw["linestyle"] = "None"
-                    l = self.ax.plot(x, y, **merge({}, lkw, line_kw))
+                        if not s.is_filled:
+                            lkw["markerfacecolor"] = (1, 1, 1)
+                    kw = merge({}, lkw, s.rendering_kw)
+                    l = self.ax.plot(x, y, **kw)
                     self._add_handle(i, l)
 
-            elif s.is_contour and (not s.is_complex):
+            elif s.is_contour:
                 x, y, z = s.get_data()
                 ckw = dict(cmap=next(self._cm))
-                contour_kw = self._kwargs.get("contour_kw", dict())
-                kw = merge({}, ckw, contour_kw)
+                kw = merge({}, ckw, s.rendering_kw)
                 c = self.ax.contourf(x, y, z, **kw)
                 self._add_colorbar(c, s.label, True)
                 self._add_handle(i, c, kw, self._fig.axes[-1])
 
-            elif s.is_contour and s.is_complex:
-                # this is specifically tailored to create cplot-like contour
-                # lines for magnitude/argument of a complex function.
-                x, y, z, r = s.get_data()
-                ckw = dict(colors="#a0a0a050", linestyles="solid")
-                contour_kw = self._kwargs.get("contour_kw", dict())
-                kw = merge({}, ckw, contour_kw)
-
-                levels = contour_kw.pop("levels", None)
-                if levels is None:
-                    if s.abs:
-                        ckw["levels"] = s.abs_levels
-                        kw = merge({}, ckw, contour_kw)
-                        c = self.ax.contour(x, y, z, **kw)
-                    else:
-                        levels = s.arg_levels
-                        if len(levels) > 0:
-                            c = self.ax.contour(
-                                x, y, s.angle_func(r), levels=levels, **kw
-                            )
-
-                            for level, allseg in zip(levels, c.allsegs):
-                                for segment in allseg:
-                                    xx, yy = segment.T
-                                    zz = xx + 1j * yy
-                                    angle = s.angle_func(s.function(zz))
-                                    # cut off segments close to the branch cut
-                                    is_near_branch_cut = np.logical_or(
-                                        *[
-                                            np.abs(angle - bc) < np.abs(angle - level)
-                                            for bc in s.angle_range
-                                        ]
-                                    )
-                                    segment[is_near_branch_cut] = np.nan
-
-                self._add_handle(i, c, kw, self._fig.axes[-1])
-
             elif s.is_3Dline:
                 x, y, z, param = s.get_data()
                 lkw = dict()
-                line_kw = self._kwargs.get("line_kw", dict())
 
                 if len(x) > 1:
                     if self._use_cm:
                         segments = self.get_segments(x, y, z)
                         lkw["cmap"] = next(self._cm)
                         lkw["array"] = param
-                        c = Line3DCollection(segments, **merge({}, lkw, line_kw))
+                        kw = merge({}, lkw, s.rendering_kw)
+                        c = Line3DCollection(segments, **kw)
                         self.ax.add_collection(c)
                         self._add_colorbar(c, s.label)
                         self._add_handle(i, c)
                     else:
                         lkw["label"] = s.label
-                        l = self.ax.plot(x, y, z, **merge({}, lkw, line_kw))
+                        kw = merge({}, lkw, s.rendering_kw)
+                        l = self.ax.plot(x, y, z, **kw)
                         self._add_handle(i, l)
                 else:
                     # 3D points
                     lkw["label"] = s.label
                     lkw["color"] = next(self._cl)
-                    l = self.ax.scatter(x, y, z, **merge({}, lkw, line_kw))
+                    kw = merge({}, lkw, s.rendering_kw)
+                    l = self.ax.scatter(x, y, z, **kw)
                     self._add_handle(i, l)
                 xlims.append((np.amin(x), np.amax(x)))
                 ylims.append((np.amin(y), np.amax(y)))
@@ -414,12 +399,10 @@ class MatplotlibBackend(Plot):
                 skw = dict(rstride=1, cstride=1, linewidth=0.1)
                 if self._use_cm:
                     skw["cmap"] = next(self._cm)
-                surface_kw = self._kwargs.get("surface_kw", dict())
-                kw = merge({}, skw, surface_kw)
+                kw = merge({}, skw, s.rendering_kw)
                 c = self.ax.plot_surface(x, y, z, **kw)
                 is_cb_added = self._add_colorbar(c, s.label)
                 self._add_handle(i, c, kw, is_cb_added, self._fig.axes[-1])
-
                 xlims.append((np.amin(x), np.amax(x)))
                 ylims.append((np.amin(y), np.amax(y)))
                 zlims.append((np.amin(z), np.amax(z)))
@@ -429,23 +412,25 @@ class MatplotlibBackend(Plot):
                 if len(points) == 2:
                     # interval math plotting
                     x, y = _matplotlib_list(points[0])
-                    c = self.ax.fill(x, y, edgecolor="None")
+                    c = self.ax.fill(x, y, color=next(self._cl),
+                        edgecolor="None")
                     self._add_handle(i, c)
                 else:
                     # use contourf or contour depending on whether it is
                     # an inequality or equality.
-                    xarray, yarray, zarray, ones, plot_type = points
+                    xarray, yarray, zarray, plot_type = points
+                    color = next(self._cl)
                     if plot_type == "contour":
-                        ckw = dict(cmap=next(self._cm))
-                        contour_kw = self._kwargs.get("contour_kw", dict())
-                        kw = merge({}, ckw, contour_kw)
-                        c = self.ax.contour(xarray, yarray, zarray, [0.0], **kw)
-                    else:
-                        colormap = ListedColormap(["#FFFFFF00", next(self._cl)])
+                        colormap = self.ListedColormap([color, color])
                         ckw = dict(cmap=colormap)
-                        contour_kw = self._kwargs.get("contour_kw", dict())
-                        kw = merge({}, ckw, contour_kw)
-                        c = self.ax.contourf(xarray, yarray, ones, **kw)
+                        kw = merge({}, ckw, s.rendering_kw)
+                        c = self.ax.contour(xarray, yarray, zarray, [0.0],
+                            **kw)
+                    else:
+                        colormap = self.ListedColormap(["white", color])
+                        ckw = dict(cmap=colormap)
+                        kw = merge({}, ckw, s.rendering_kw)
+                        c = self.ax.contourf(xarray, yarray, zarray, **kw)
                     self._add_handle(i, c, kw)
 
             elif s.is_vector:
@@ -454,67 +439,65 @@ class MatplotlibBackend(Plot):
                     magn = np.sqrt(uu ** 2 + vv ** 2)
                     if s.is_streamlines:
                         skw = dict()
-                        stream_kw = self._kwargs.get("stream_kw", dict())
-                        if self._use_cm:
+                        if (not s.use_quiver_solid_color) and self._use_cm:
                             skw["cmap"] = next(self._cm)
                             skw["color"] = magn
+                            kw = merge({}, skw, s.rendering_kw)
+                            sp = self.ax.streamplot(xx, yy, uu, vv, **kw)
+                            is_cb_added = self._add_colorbar(sp.lines, s.label)
                         else:
                             skw["color"] = next(self._cl)
-                        kw = merge({}, skw, stream_kw)
-                        s = self.ax.streamplot(xx, yy, uu, vv, **kw)
-                        self._add_handle(i, s, kw)
+                            kw = merge({}, skw, s.rendering_kw)
+                            sp = self.ax.streamplot(xx, yy, uu, vv, **kw)
+                            is_cb_added = False
+                        self._add_handle(i, sp, kw, is_cb_added,
+                            self._fig.axes[-1])
                     else:
                         qkw = dict()
-                        quiver_kw = self._kwargs.get("quiver_kw", dict())
-                        # don't use color map if a scalar field is visible or if
-                        # use_cm=False
-                        solid = (
-                            True
-                            if ("scalar" not in self._kwargs.keys())
-                            else (
-                                False
-                                if (
-                                    (not self._kwargs["scalar"])
-                                    or (self._kwargs["scalar"] is None)
-                                )
-                                else True
-                            )
-                        )
-                        if (not solid) and self._use_cm:
+                        if (not s.use_quiver_solid_color) and self._use_cm:
+                            # don't use color map if a scalar field is
+                            # visible or if use_cm=False
                             qkw["cmap"] = next(self._cm)
-                            kw = merge({}, qkw, quiver_kw)
+                            kw = merge({}, qkw, s.rendering_kw)
                             q = self.ax.quiver(xx, yy, uu, vv, magn, **kw)
                             is_cb_added = self._add_colorbar(q, s.label)
                         else:
                             is_cb_added = False
                             qkw["color"] = next(self._cl)
-                            kw = merge({}, qkw, quiver_kw)
+                            kw = merge({}, qkw, s.rendering_kw)
                             q = self.ax.quiver(xx, yy, uu, vv, **kw)
-                        self._add_handle(i, q, kw, is_cb_added, self._fig.axes[-1])
+                        self._add_handle(i, q, kw, is_cb_added,
+                            self._fig.axes[-1])
                 else:
                     xx, yy, zz, uu, vv, ww = s.get_data()
                     magn = np.sqrt(uu ** 2 + vv ** 2 + ww ** 2)
 
                     if s.is_streamlines:
                         vertices, magn = compute_streamtubes(
-                            xx, yy, zz, uu, vv, ww, self._kwargs)
+                            xx, yy, zz, uu, vv, ww, s.rendering_kw)
 
                         lkw = dict()
-                        line_kw = self._kwargs.get("line_kw", dict())
+                        stream_kw = s.rendering_kw.copy()
+                        # remove rendering-unrelated keywords
+                        for k in ["starts", "max_prop", "npoints", "radius"]:
+                            if k in stream_kw.keys():
+                                stream_kw.pop(k)
 
                         if self._use_cm:
                             segments = self.get_segments(
                                 vertices[:, 0], vertices[:, 1], vertices[:, 2])
                             lkw["cmap"] = next(self._cm)
                             lkw["array"] = magn
-                            c = Line3DCollection(segments, **merge({}, lkw, line_kw))
+                            kw = merge({}, lkw, stream_kw)
+                            c = Line3DCollection(segments, **kw)
                             self.ax.add_collection(c)
                             self._add_colorbar(c, s.label)
                             self._add_handle(i, c)
                         else:
                             lkw["label"] = s.label
+                            kw = merge({}, lkw, stream_kw)
                             l = self.ax.plot(vertices[:, 0], vertices[:, 1],
-                                vertices[:, 2], **merge({}, lkw, line_kw))
+                                vertices[:, 2], **kw)
                             self._add_handle(i, l)
 
                         xlims.append((np.amin(xx), np.amax(xx)))
@@ -522,16 +505,15 @@ class MatplotlibBackend(Plot):
                         zlims.append((np.amin(zz), np.amax(zz)))
                     else:
                         qkw = dict()
-                        quiver_kw = self._kwargs.get("quiver_kw", dict())
                         if self._use_cm:
                             qkw["cmap"] = next(self._cm)
                             qkw["array"] = magn.flatten()
-                            kw = merge({}, qkw, quiver_kw)
+                            kw = merge({}, qkw, s.rendering_kw)
                             q = self.ax.quiver(xx, yy, zz, uu, vv, ww, **kw)
                             is_cb_added = self._add_colorbar(q, s.label)
                         else:
                             qkw["color"] = next(self._cl)
-                            kw = merge({}, qkw, quiver_kw)
+                            kw = merge({}, qkw, s.rendering_kw)
                             q = self.ax.quiver(xx, yy, zz, uu, vv, ww, **kw)
                             is_cb_added = False
                         self._add_handle(i, q, kw, is_cb_added, self._fig.axes[-1])
@@ -541,15 +523,13 @@ class MatplotlibBackend(Plot):
 
             elif s.is_complex:
                 if not s.is_3Dsurface:
-                    # x, y, magn_angle, img, discr, colors = self._get_image(s)
                     x, y, _, _, img, colors = s.get_data()
-                    image_kw = self._kwargs.get("image_kw", dict())
                     ikw = dict(
                         extent=[np.amin(x), np.amax(x), np.amin(y), np.amax(y)],
                         interpolation="nearest",
                         origin="lower",
                     )
-                    kw = merge({}, ikw, image_kw)
+                    kw = merge({}, ikw, s.rendering_kw)
                     image = self.ax.imshow(img, **kw)
                     self._add_handle(i, image, kw)
 
@@ -557,10 +537,10 @@ class MatplotlibBackend(Plot):
                     if colors is not None:
                         colors = colors / 255.0
 
-                        colormap = ListedColormap(colors)
-                        norm = Normalize(vmin=-np.pi, vmax=np.pi)
+                        colormap = self.ListedColormap(colors)
+                        norm = self.Normalize(vmin=-np.pi, vmax=np.pi)
                         cb2 = self._fig.colorbar(
-                            cm.ScalarMappable(norm=norm, cmap=colormap),
+                            self.cm.ScalarMappable(norm=norm, cmap=colormap),
                             orientation="vertical",
                             label="Argument",
                             ticks=[-np.pi, -np.pi / 2, 0, np.pi / 2, np.pi],
@@ -575,8 +555,7 @@ class MatplotlibBackend(Plot):
                     skw = dict(rstride=1, cstride=1, linewidth=0.1)
                     if self._use_cm:
                         skw["facecolors"] = facecolors / 255
-                    surface_kw = self._kwargs.get("surface_kw", dict())
-                    kw = merge({}, skw, surface_kw)
+                    kw = merge({}, skw, s.rendering_kw)
                     c = self.ax.plot_surface(x, y, mag, **kw)
 
                     if self._use_cm and (colorscale is not None):
@@ -587,9 +566,9 @@ class MatplotlibBackend(Plot):
 
                         # this colorbar is essential to understand the plot.
                         # Always show it, except when use_cm=False
-                        norm = Normalize(vmin=-np.pi, vmax=np.pi)
-                        mappable = cm.ScalarMappable(
-                            cmap=ListedColormap(colorscale), norm=norm
+                        norm = self.Normalize(vmin=-np.pi, vmax=np.pi)
+                        mappable = self.cm.ScalarMappable(
+                            cmap=self.ListedColormap(colorscale), norm=norm
                         )
                         cb = self._fig.colorbar(
                             mappable,
@@ -602,7 +581,6 @@ class MatplotlibBackend(Plot):
                             [r"-$\pi$", r"-$\pi / 2$", "0", r"$\pi / 2$", r"$\pi$"]
                         )
                     self._add_handle(i, c, kw)
-
                     xlims.append((np.amin(x), np.amax(x)))
                     ylims.append((np.amin(y), np.amax(y)))
                     zlims.append((np.amin(mag), np.amax(mag)))
@@ -610,9 +588,8 @@ class MatplotlibBackend(Plot):
             elif s.is_geometry:
                 x, y = s.get_data()
                 color = next(self._cl)
-                fkw = dict(facecolor=color, fill=s.fill, edgecolor=color)
-                fill_kw = self._kwargs.get("fill_kw", dict())
-                kw = merge({}, fkw, fill_kw)
+                fkw = dict(facecolor=color, fill=s.is_filled, edgecolor=color)
+                kw = merge({}, fkw, s.rendering_kw)
                 c = self.ax.fill(x, y, **kw)
                 self._add_handle(i, c, kw)
 
@@ -660,7 +637,19 @@ class MatplotlibBackend(Plot):
                 self.ax.spines["right"].set_visible(False)
                 self.ax.spines["top"].set_visible(False)
         if self.grid:
-            self.ax.grid()
+            if isinstance(self.ax, Axes3D):
+                self.ax.grid()
+            else:
+                self.ax.grid(which='major', axis='x', linewidth=0.75,
+                    linestyle='-', color='0.85')
+                self.ax.grid(which='minor', axis='x', linewidth=0.25,
+                    linestyle='--', color='0.80')
+                self.ax.grid(which='major', axis='y', linewidth=0.75,
+                    linestyle='-', color='0.85')
+                self.ax.grid(which='minor', axis='y', linewidth=0.25,
+                    linestyle='--', color='0.80')
+                if self._show_minor_grid:
+                    self.ax.minorticks_on()
         if self.legend:
             handles, _ = self.ax.get_legend_handles_labels()
             # Show the legend only if there are legend entries. For example,
@@ -684,13 +673,18 @@ class MatplotlibBackend(Plot):
         self._set_lims(xlims, ylims, zlims)
 
     def _set_lims(self, xlims, ylims, zlims):
+        np = import_module('numpy')
+        mpl_toolkits = import_module(
+            'mpl_toolkits', # noqa
+            import_kwargs={'fromlist': ['mplot3d']},
+            catch=(RuntimeError,))
         Axes3D = mpl_toolkits.mplot3d.Axes3D
         if not isinstance(self.ax, Axes3D):
             self.ax.autoscale_view(
                 scalex=self.ax.get_autoscalex_on(), scaley=self.ax.get_autoscaley_on()
             )
 
-            # NOTE: Hack in order to make interactive contour plots to scale to
+            # HACK: in order to make interactive contour plots to scale to
             # the appropriate range
             if xlims and (
                 any(s.is_contour for s in self.series)
@@ -752,14 +746,22 @@ class MatplotlibBackend(Plot):
         The name is misleading: here we create a new colorbar which will be
         placed on the same colorbar axis as the original.
         """
+        np = import_module('numpy')
         cax.clear()
-        norm = Normalize(vmin=np.amin(param), vmax=np.amax(param))
-        mappable = cm.ScalarMappable(cmap=kw["cmap"], norm=norm)
+        norm = self.Normalize(vmin=np.amin(param), vmax=np.amax(param))
+        mappable = self.cm.ScalarMappable(cmap=kw["cmap"], norm=norm)
         self._fig.colorbar(mappable, orientation="vertical", label=label, cax=cax)
 
     def _update_interactive(self, params):
-        # With this backend, data is only being added once the plot is shown.
-        # However, iplot doesn't call the show method. The following line of
+        np = import_module('numpy')
+        mpl_toolkits = import_module(
+            'mpl_toolkits', # noqa
+            import_kwargs={'fromlist': ['mplot3d']},
+            catch=(RuntimeError,))
+        Line3DCollection = mpl_toolkits.mplot3d.art3d.Line3DCollection
+        Path3DCollection = mpl_toolkits.mplot3d.art3d.Path3DCollection
+
+        # iplot doesn't call the show method. The following line of
         # code will add the numerical data (if not already present).
         if len(self._handles) == 0:
             self.process_series()
@@ -767,7 +769,7 @@ class MatplotlibBackend(Plot):
         xlims, ylims, zlims = [], [], []
         for i, s in enumerate(self.series):
             if s.is_interactive:
-                self.series[i].update_data(params)
+                self.series[i].params = params
                 if s.is_2Dline:
                     if s.is_parametric and self._use_cm:
                         x, y, param = self.series[i].get_data()
@@ -784,8 +786,7 @@ class MatplotlibBackend(Plot):
                             x, y, param = self.series[i].get_data()
                         else:
                             x, y = self.series[i].get_data()
-                            x, y, _ = self._detect_poles(x, y)
-                        # TODO: Point2D are update but not visible.
+                        # TODO: Point2D are updated but not visible.
                         self._handles[i][0].set_data(x, y)
 
                 elif s.is_3Dline:
@@ -810,20 +811,18 @@ class MatplotlibBackend(Plot):
                     for c in self._handles[i][0].collections:
                         c.remove()
                     self._handles[i][0] = self.ax.contourf(x, y, z, **kw)
-
                     self._update_colorbar(cax, z, kw, s.label)
                     xlims.append((np.amin(x), np.amax(x)))
                     ylims.append((np.amin(y), np.amax(y)))
 
                 elif s.is_3Dsurface and (not s.is_domain_coloring):
                     x, y, z = self.series[i].get_data()
-                    # TODO: by setting the keyword arguments, somehow the update
-                    # becomes really really slow.
+                    # TODO: by setting the keyword arguments, somehow the
+                    # update becomes really really slow.
                     kw, is_cb_added, cax = self._handles[i][1:]
                     self._handles[i][0].remove()
                     self._handles[i][0] = self.ax.plot_surface(
-                        x, y, z, **self._handles[i][1]
-                    )
+                        x, y, z, **kw)
 
                     if is_cb_added:
                         self._update_colorbar(cax, z, kw, s.label)
@@ -871,9 +870,9 @@ class MatplotlibBackend(Plot):
                     if s.is_streamlines:
                         raise NotImplementedError
 
-                        # TODO: streamlines are composed by lines and arrows.
+                        # Streamlines are composed by lines and arrows.
                         # Arrows belongs to a PatchCollection. Currently,
-                        # there is no way to remove a PatchCollectio....
+                        # there is no way to remove a PatchCollection....
                         kw = self._handles[i][1]
                         self._handles[i][0].lines.remove()
                         self._handles[i][0].arrows.remove()
@@ -898,12 +897,9 @@ class MatplotlibBackend(Plot):
                         x, y, mag, arg, facecolors, colorscale = s.get_data()
                         self._handles[i][0].remove()
                         kw = self._handles[i][1]
-
                         if self._use_cm:
                             kw["facecolors"] = facecolors / 255
-
                         self._handles[i][0] = self.ax.plot_surface(x, y, mag, **kw)
-
                         xlims.append((np.amin(x), np.amax(x)))
                         ylims.append((np.amin(y), np.amax(y)))
                         zlims.append((np.amin(mag), np.amax(mag)))
@@ -928,9 +924,8 @@ class MatplotlibBackend(Plot):
         self._set_lims(xlims, ylims, zlims)
 
     def process_series(self):
-        """
-        Iterates over every ``Plot`` object and further calls
-        _process_series()
+        """ Loop over data series, generates numerical data and add it to the
+        figure.
         """
         # create the figure from scratch every time, otherwise if the plot was
         # previously shown, it would not be possible to show it again. This
@@ -943,7 +938,7 @@ class MatplotlibBackend(Plot):
         self.process_series()
         if _show:
             self._fig.tight_layout()
-            self._fig.show()
+            self.plt.show()
         else:
             self.close()
 
@@ -956,15 +951,13 @@ class MatplotlibBackend(Plot):
         ==========
         .. [#fn8] https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.savefig.html
         """
-        # NOTE: the plot must first be created and then saved. Turns out that in
-        # in the process the plot will also be shown. That's standard Matplotlib
-        # behaviour.
-        self.show()
+        if self._fig is None:
+            self.process_series()
         self._fig.savefig(path, **kwargs)
 
     def close(self):
         """Close the current plot."""
-        plt.close(self._fig)
+        self.plt.close(self._fig)
 
 
 MB = MatplotlibBackend
