@@ -1,24 +1,11 @@
 from spb.defaults import cfg
 from spb.backends.base_backend import Plot
 from spb.backends.utils import get_seeds_points
-import plotly.graph_objects as go
-from plotly.figure_factory import create_quiver, create_streamline
-from mergedeep import merge
+from sympy.external import import_module
 import itertools
-import matplotlib.cm as cm
-from spb.backends.utils import convert_colormap
 import warnings
-import numpy as np
 import os
 
-"""
-TODO:
-1. iplot support for 2D/3D streamlines.
-2. iplot support for domain coloring.
-3. custom surface color for 3D complex plots.
-
-"""
-
 
 class PlotlyBackend(Plot):
     """
@@ -48,9 +35,10 @@ class PlotlyBackend(Plot):
         A dictionary of keywords/values which is passed to Plotly's quivers
         function to customize the appearance.
 
-        - For 2D vector fields, default to: ``dict( scale = 0.075 )``
+        - For 2D vector fields, default to: ``dict( scale = 0.075 )``.
           Refer to [#fn5]_ for more options.
-        - For 3D vector fields, default to: ``dict( sizemode = "absolute", sizeref = 40 )``
+        - For 3D vector fields, default to:
+          ``dict( sizemode = "absolute", sizeref = 40 )``.
           Refer [#fn6]_ for more options.
 
     surface_kw : dict, optional
@@ -62,9 +50,9 @@ class PlotlyBackend(Plot):
         A dictionary of keywords/values which is passed to Plotly's
         streamlines function to customize the appearance.
 
-        - For 2D vector fields, defaul to: ``dict( arrow_scale = 0.15 )``
+        - For 2D vector fields, defaul to: ``dict( arrow_scale = 0.15 )``.
           Refer to [#fn8]_ for more options.
-        - For 3D vector fields, default to: ``dict( sizeref = 0.3 )``
+        - For 3D vector fields, default to: ``dict( sizeref = 0.3 )``.
           Refer to [#fn9]_ for more options.
 
     theme : str, optional
@@ -75,11 +63,6 @@ class PlotlyBackend(Plot):
         If True, apply a color map to the meshes/surface. If False, solid
         colors will be used instead. Default to True.
 
-    wireframe : boolean, optional
-        Visualize the wireframe lines on surfaces. Default to False.
-        Note that it definitely have a negative impact on the performances, and
-        it does not work with ``iplot``.
-    
     References
     ==========
     .. [#fn1] https://plotly.com/python/contour-plots/
@@ -92,8 +75,20 @@ class PlotlyBackend(Plot):
     .. [#fn8] https://plotly.com/python/streamline-plots/
     .. [#fn9] https://plotly.com/python/streamtube-plot/
     .. [#fn10] https://plotly.com/python/templates/
+    .. [#fn13] https://github.com/plotly/plotly.js/issues/5003
 
 
+    Notes
+    =====
+
+    A few bugs related to Plotly might prevent the correct visualization:
+
+    * with 2D domain coloring, the vertical axis is reversed, with negative
+      values on the top and positive values on the bottom.
+    * with 3D complex plots: when hovering a point, the tooltip will display
+      wrong information for the argument and the phase. Hopefully, this bug
+      [#fn13]_ will be fixed upstream.
+
     See also
     ========
 
@@ -102,75 +97,10 @@ class PlotlyBackend(Plot):
 
     _library = "plotly"
 
-    # The following colors corresponds to the discret color map
-    # px.colors.qualitative.Plotly. They are here in order to avoid the
-    # following statement: import plotly.express as px
-    # which happens to slow down the loading phase.
-    colorloop = [
-        "#636EFA",
-        "#EF553B",
-        "#00CC96",
-        "#AB63FA",
-        "#FFA15A",
-        "#19D3F3",
-        "#FF6692",
-        "#B6E880",
-        "#FF97FF",
-        "#FECB52",
-    ]
-    # a selection of color maps to be used in 3D surfaces, contour plots.
-    # NOTE: if you change any of these, make sure to also change
-    # wireframe_colors.
-    colormaps = [
-        "aggrnyl",
-        "plotly3",
-        "reds_r",
-        "ice",
-        "inferno",
-        "deep_r",
-        "turbid_r",
-        "gnbu_r",
-        "geyser_r",
-        "oranges_r",
-    ]
-
-    # to be used in complex-parametric plots
-    cyclic_colormaps = ["phase", "twilight", "hsv", "icefire"]
-
-    # a selection of solid colors for wireframe lines that may look good with
-    # the above colormaps
-    wireframe_colors = [
-        "#0071c3",
-        "#af67d9",
-        "#e64b17",
-        "#1378cd",
-        "#be5466",
-        "#6f969e",
-        "#aa692c",
-        "#60ccc0",
-        "#f2a45d",
-        "#f2a45d",
-    ]
-    # a few solid color that offers medium to good contrast against Plotly's
-    # default colorscale for contour plots
-    # TODO: here I selected black and white, but they are not visible with dark
-    # or light theme respectively... Need a better selection of colors.
-    # Although, they are placed in the middle of the loop, so they are unlikely
-    # going to be used.
-    quivers_colors = [
-        "magenta",
-        "crimson",
-        "darkorange",
-        "dodgerblue",
-        "wheat",
-        "slategrey",
-        "white",
-        "black",
-        "darkred",
-        "indigo"
-        # "cyan", "greenyellow", "grey", "darkred", "white",
-        # "black", "orange", "silver", "darkcyan", "magenta"
-    ]
+    colorloop = []
+    colormaps = []
+    cyclic_colormaps = []
+    quivers_colors = []
 
     # color bar spacing
     _cbs = 0.15
@@ -178,67 +108,87 @@ class PlotlyBackend(Plot):
     _cbsdf = 0.75
 
     def __new__(cls, *args, **kwargs):
-        # Since Plot has its __new__ method, this will prevent infinite
-        # recursion
         return object.__new__(cls)
 
     def __init__(self, *args, **kwargs):
+        plotly = import_module(
+            'plotly',
+            import_kwargs={'fromlist': ['graph_objects', 'figure_factory']},
+            min_module_version='5.0.0')
+        go = plotly.graph_objects
+
+        # The following colors corresponds to the discret color map
+        # px.colors.qualitative.Plotly.
+        self.colorloop = [
+            "#636EFA", "#EF553B", "#00CC96", "#AB63FA", "#FFA15A",
+            "#19D3F3", "#FF6692", "#B6E880", "#FF97FF", "#FECB52"]
+        self.colormaps = [
+            "aggrnyl", "plotly3", "reds_r", "ice", "inferno",
+            "deep_r", "turbid_r", "gnbu_r", "geyser_r", "oranges_r"]
+        self.cyclic_colormaps = ["phase", "twilight", "hsv", "icefire"]
+        # TODO: here I selected black and white, but they are not visible
+        # with dark or light theme respectively... Need a better selection
+        # of colors. Although, they are placed in the middle of the loop,
+        # so they are unlikely going to be used.
+        self.quivers_colors = [
+            "magenta", "crimson", "darkorange", "dodgerblue", "wheat",
+            "slategrey", "white", "black", "darkred", "indigo"]
+
+        self._init_cyclers()
         super().__init__(*args, **kwargs)
 
-        # add colors if needed
-        default_cl = [
-            "#636EFA",
-            "#EF553B",
-            "#00CC96",
-            "#AB63FA",
-            "#FFA15A",
-            "#19D3F3",
-            "#FF6692",
-            "#B6E880",
-            "#FF97FF",
-            "#FECB52",
-        ]
-        if (len([s for s in self._series if s.is_2Dline]) > 10) and (
-            self.colorloop == default_cl
-        ):
+        if ((len([s for s in self._series if s.is_2Dline]) > 10) and
+            (not type(self).colorloop) and
+            not ("process_piecewise" in kwargs.keys())):
+            # add colors if needed
             # this corresponds to px.colors.qualitative.Light24
             self.colorloop = [
-                "#FD3216",
-                "#00FE35",
-                "#6A76FC",
-                "#FED4C4",
-                "#FE00CE",
-                "#0DF9FF",
-                "#F6F926",
-                "#FF9616",
-                "#479B55",
-                "#EEA6FB",
-                "#DC587D",
-                "#D626FF",
-                "#6E899C",
-                "#00B5F7",
-                "#B68E00",
-                "#C9FBE5",
-                "#FF0092",
-                "#22FFA7",
-                "#E3EE9E",
-                "#86CE00",
-                "#BC7196",
-                "#7E7DCD",
-                "#FC6955",
-                "#E48F72",
+                "#FD3216", "#00FE35", "#6A76FC", "#FED4C4", "#FE00CE",
+                "#0DF9FF", "#F6F926", "#FF9616", "#479B55", "#EEA6FB",
+                "#DC587D", "#D626FF", "#6E899C", "#00B5F7", "#B68E00",
+                "#C9FBE5", "#FF0092", "#22FFA7", "#E3EE9E", "#86CE00",
+                "#BC7196", "#7E7DCD", "#FC6955", "#E48F72"
             ]
 
+        self._theme = kwargs.get("theme", cfg["plotly"]["theme"])
         self._fig = go.Figure()
+
+    @property
+    def fig(self):
+        """Returns the figure."""
+        if len(self.series) != len(self._fig.data):
+            # if the backend was created without showing it
+            self.process_series()
+        return self._fig
+
+    def process_series(self):
+        """ Loop over data series, generates numerical data and add it to the
+        figure.
+        """
         # this is necessary in order for the series to be added even if
         # show=False
         self._process_series(self._series)
         self._update_layout()
 
+    def _set_piecewise_color(self, s, color):
+        """Set the color to the given series"""
+        s.rendering_kw["line_color"] = color
+        if not s.is_filled:
+            s.rendering_kw["marker"] = dict(
+                color="#E5ECF6",
+                line=dict(color=color))
+
+    @staticmethod
+    def _do_sum_kwargs(p1, p2):
+        kw = p1._copy_kwargs()
+        kw["theme"] = p1._theme
+        return kw
+
     def _init_cyclers(self):
         super()._init_cyclers()
-        self._wfcm = itertools.cycle(self.wireframe_colors)
-        self._qc = itertools.cycle(self.quivers_colors)
+        tb = type(self)
+        quivers_colors = self.quivers_colors if not tb.quivers_colors else tb.quivers_colors
+        self._qc = itertools.cycle(quivers_colors)
 
     def _create_colorbar(self, k, label, sc=False):
         """This method reduces code repetition.
@@ -269,6 +219,15 @@ class PlotlyBackend(Plot):
         return [[0, col], [1, col]]
 
     def _process_series(self, series):
+        np = import_module('numpy')
+        plotly = import_module(
+            'plotly',
+            import_kwargs={'fromlist': ['graph_objects', 'figure_factory']},
+            min_module_version='5.0.0')
+        go = plotly.graph_objects
+        create_quiver = plotly.figure_factory.create_quiver
+        create_streamline = plotly.figure_factory.create_streamline
+        merge = self.merge
         self._init_cyclers()
 
         # if legend=True and both 3d lines and surfaces are shown, then hide the
@@ -287,8 +246,9 @@ class PlotlyBackend(Plot):
 
         count = 0
         for ii, s in enumerate(series):
+            kw = None
+
             if s.is_2Dline:
-                line_kw = self._kwargs.get("line_kw", dict())
                 if s.is_parametric:
                     x, y, param = s.get_data()
                     # hides/show the colormap depending on self._use_cm
@@ -317,16 +277,29 @@ class PlotlyBackend(Plot):
                             else "x: %{x}<br />y: %{y}<br />Arg: %{customdata}"
                         ),
                     )
-                    self._fig.add_trace(go.Scatter(x=x, y=y, **merge({}, lkw, line_kw)))
+                    kw = merge({}, lkw, s.rendering_kw)
+                    self._fig.add_trace(go.Scatter(x=x, y=y, **kw))
                 else:
                     x, y = s.get_data()
-                    x, y, _ = self._detect_poles(x, y)
+                    color = next(self._cl)
                     lkw = dict(
                         name=s.label,
                         mode="lines" if not s.is_point else "markers",
-                        line_color=next(self._cl),
+                        line_color=color
                     )
-                    self._fig.add_trace(go.Scatter(x=x, y=y, **merge({}, lkw, line_kw)))
+                    if s.is_point:
+                        lkw["marker"] = dict(size=8)
+                        if not s.is_filled:
+                            lkw["marker"] = dict(
+                                color="#E5ECF6",
+                                size=8,
+                                line=dict(
+                                    width=2,
+                                    color=color
+                                )
+                            )
+                    kw = merge({}, lkw, s.rendering_kw)
+                    self._fig.add_trace(go.Scatter(x=x, y=y, **kw))
             elif s.is_3Dline:
                 # NOTE: As a design choice, I decided to show the legend entry
                 # as well as the colorbar (if use_cm=True). Even though the
@@ -355,10 +328,8 @@ class PlotlyBackend(Plot):
                         name=s.label,
                         mode="markers",
                         line_color=next(self._cl))
-                line_kw = self._kwargs.get("line_kw", dict())
-                self._fig.add_trace(
-                    go.Scatter3d(x=x, y=y, z=z, **merge({}, lkw, line_kw))
-                )
+                kw = merge({}, lkw, s.rendering_kw)
+                self._fig.add_trace(go.Scatter3d(x=x, y=y, z=z, **kw))
 
             elif (s.is_3Dsurface and not s.is_domain_coloring):
                 xx, yy, zz = s.get_data()
@@ -374,43 +345,15 @@ class PlotlyBackend(Plot):
                     colorscale=colormap if self._use_cm else colorscale,
                 )
 
-                surface_kw = self._kwargs.get("surface_kw", dict())
-                self._fig.add_trace(
-                    go.Surface(x=xx, y=yy, z=zz, **merge({}, skw, surface_kw))
-                )
+                kw = merge({}, skw, s.rendering_kw)
+                self._fig.add_trace(go.Surface(x=xx, y=yy, z=zz, **kw))
 
-                # TODO: remove this? Making it works with iplot is difficult
-                if self._kwargs.get("wireframe", False):
-                    line_marker = dict(color=next(self._wfcm), width=2)
-                    for i, j, k in zip(xx, yy, zz):
-                        self._fig.add_trace(
-                            go.Scatter3d(
-                                x=i,
-                                y=j,
-                                z=k,
-                                mode="lines",
-                                line=line_marker,
-                                showlegend=False,
-                            )
-                        )
-                    for i, j, k in zip(xx.T, yy.T, zz.T):
-                        self._fig.add_trace(
-                            go.Scatter3d(
-                                x=i,
-                                y=j,
-                                z=k,
-                                mode="lines",
-                                line=line_marker,
-                                showlegend=False,
-                            )
-                        )
                 count += 1
 
             elif s.is_contour and (not s.is_complex):
                 xx, yy, zz = s.get_data()
                 xx = xx[0, :]
                 yy = yy[:, 0]
-                # default values
                 ckw = dict(
                     contours=dict(
                         coloring=None,
@@ -419,98 +362,36 @@ class PlotlyBackend(Plot):
                     colorscale=next(self._cm),
                     colorbar=self._create_colorbar(ii, s.label, show_2D_vectors),
                 )
-                # user-provided values
-                contour_kw = self._kwargs.get("contour_kw", dict())
-                self._fig.add_trace(
-                    go.Contour(x=xx, y=yy, z=zz, **merge({}, ckw, contour_kw))
-                )
-                count += 1
-
-            elif s.is_implicit:
-                points = s.get_data()
-                if len(points) == 2:
-                    # interval math plotting
-                    x, y, pixels = self._get_pixels(s, points[0])
-                    ckw = dict(
-                        colorscale=[
-                            [0, "rgba(0,0,0,0)"],
-                            [1, next(self._cl)],
-                        ],
-                        showscale=False,
-                        name=s.label,
-                    )
-                    contour_kw = self._kwargs.get("contour_kw", dict())
-                    self._fig.add_trace(
-                        go.Heatmap(x=x, y=y, z=pixels, **merge({}, ckw, contour_kw))
-                    )
-                else:
-                    x, y, z, ones, plot_type = points
-                    zf = z.flatten()
-                    m, M = min(zf), max(zf)
-                    col = next(self._cl)
-                    # default values
-                    ckw = dict(
-                        contours=dict(
-                            coloring="none" if plot_type == "contour" else None,
-                            showlabels=False,
-                            type="levels" if plot_type == "contour" else "constraint",
-                            operation="<",
-                            value=[(m + M) / 2],
-                        ),
-                        colorscale=[
-                            [0, "rgba(0,0,0,0)"],
-                            [1, next(self._cl)],
-                        ],
-                        fillcolor=col,
-                        showscale=True,
-                        name=s.label,
-                        line=dict(color=col),
-                    )
-                    contour_kw = self._kwargs.get("contour_kw", dict())
-                    # TODO: sadly, Plotly does not support yet setting contour
-                    # levels, hence the visualization will look ugly whenever
-                    # plot_type="contour".
-                    # https://github.com/plotly/plotly.js/issues/4503
-                    self._fig.add_trace(
-                        go.Contour(x=x, y=y, z=ones, **merge({}, ckw, contour_kw))
-                    )
+                kw = merge({}, ckw, s.rendering_kw)
+                self._fig.add_trace(go.Contour(x=xx, y=yy, z=zz, **kw))
                 count += 1
 
             elif s.is_vector:
                 if s.is_2Dvector:
                     xx, yy, uu, vv = s.get_data()
+                    # NOTE: currently, it is not possible to create
+                    # quivers/streamlines with a color scale:
+                    # https://community.plotly.com/t/how-to-make-python-quiver-with-colorscale/41028
                     if s.is_streamlines:
-                        # NOTE: currently, it is not possible to create streamlines with
-                        # a color scale: https://community.plotly.com/t/how-to-make-python-quiver-with-colorscale/41028
-                        # default values
                         skw = dict(
                             line_color=next(self._qc), arrow_scale=0.15, name=s.label
                         )
-                        # user-provided values
-                        stream_kw = self._kwargs.get("stream_kw", dict())
+                        kw = merge({}, skw, s.rendering_kw)
                         stream = create_streamline(
-                            xx[0, :], yy[:, 0], uu, vv, **merge({}, skw, stream_kw)
-                        )
+                            xx[0, :], yy[:, 0], uu, vv, **kw)
                         self._fig.add_trace(stream.data[0])
                     else:
-                        # NOTE: currently, it is not possible to create quivers with
-                        # a color scale: https://community.plotly.com/t/how-to-make-python-quiver-with-colorscale/41028
-                        # default values
                         qkw = dict(line_color=next(self._qc), scale=0.075, name=s.label)
-                        # user-provided values
-                        quiver_kw = self._kwargs.get("quiver_kw", dict())
-                        quiver = create_quiver(
-                            xx, yy, uu, vv, **merge({}, qkw, quiver_kw)
-                        )  # merge two dictionaries
+                        kw = merge({}, qkw, s.rendering_kw)
+                        quiver = create_quiver(xx, yy, uu, vv, **kw)
                         self._fig.add_trace(quiver.data[0])
                 else:
                     xx, yy, zz, uu, vv, ww = s.get_data()
                     if s.is_streamlines:
-                        stream_kw = self._kwargs.get("stream_kw", dict()).copy()
+                        stream_kw = s.rendering_kw.copy()
                         seeds_points = get_seeds_points(
                             xx, yy, zz, uu, vv, ww, to_numpy=True, **stream_kw)
 
-                        # default values
                         skw = dict(
                             colorscale=(
                                 next(self._cm)
@@ -527,13 +408,13 @@ class PlotlyBackend(Plot):
                             ),
                         )
 
-                        # the following keywords must be removed otherwise
-                        # Plotly's Stramtube raises error
-                        _kws = ["starts", "max_prop", "npoints", "radius"]
-                        for _k in _kws:
+                        # remove rendering-unrelated keywords
+                        for _k in ["starts", "max_prop", "npoints", "radius"]:
                             if _k in stream_kw.keys():
                                 stream_kw.pop(_k)
 
+                        kw = merge({}, skw, stream_kw)
+
                         self._fig.add_trace(
                             go.Streamtube(
                                 x=xx.flatten(),
@@ -542,11 +423,8 @@ class PlotlyBackend(Plot):
                                 u=uu.flatten(),
                                 v=vv.flatten(),
                                 w=ww.flatten(),
-                                **merge({}, skw, stream_kw)
-                            )
-                        )
+                                **kw))
                     else:
-                        # default values
                         qkw = dict(
                             showscale=(not s.is_slice) or self.legend,
                             colorscale=next(self._cm),
@@ -554,8 +432,7 @@ class PlotlyBackend(Plot):
                             sizeref=40,
                             colorbar=self._create_colorbar(ii, s.label),
                         )
-                        # user-provided values
-                        quiver_kw = self._kwargs.get("quiver_kw", dict())
+                        kw = merge({}, qkw, s.rendering_kw)
                         self._fig.add_trace(
                             go.Cone(
                                 x=xx.flatten(),
@@ -564,9 +441,7 @@ class PlotlyBackend(Plot):
                                 u=uu.flatten(),
                                 v=vv.flatten(),
                                 w=ww.flatten(),
-                                **merge({}, qkw, quiver_kw)
-                            )
-                        )
+                                **kw))
                 count += 1
 
             elif s.is_complex:
@@ -658,14 +533,14 @@ class PlotlyBackend(Plot):
                             x=1 + 0.1 * count,
                             title="Argument",
                             titleside="right",
-                            tickvals = [
+                            tickvals=[
                                 -np.pi,
                                 -np.pi / 2,
                                 0,
                                 np.pi / 2,
                                 np.pi,
                             ],
-                            ticktext = [
+                            ticktext=[
                                 "-&#x3C0;",
                                 "-&#x3C0; / 2",
                                 "0",
@@ -673,19 +548,16 @@ class PlotlyBackend(Plot):
                                 "&#x3C0;",
                             ]
                         ),
-                        cmin = -np.pi,
-                        cmax = np.pi,
+                        cmin=-np.pi,
+                        cmax=np.pi,
                         colorscale=colorscale,
-                        # colorscale=colormap if self._use_cm else colorscale,
                         surfacecolor=angle,
                         customdata=angle,
                         hovertemplate="x: %{x}<br />y: %{y}<br />Abs: %{z}<br />Arg: %{customdata}",
                     )
 
-                    surface_kw = self._kwargs.get("surface_kw", dict())
-                    self._fig.add_trace(
-                        go.Surface(x=xx, y=yy, z=mag, **merge({}, skw, surface_kw))
-                    )
+                    kw = merge({}, skw, s.rendering_kw)
+                    self._fig.add_trace(go.Surface(x=xx, y=yy, z=mag, **kw))
 
                     count += 1
 
@@ -694,8 +566,8 @@ class PlotlyBackend(Plot):
                 lkw = dict(
                     name=s.label, mode="lines", fill="toself", line_color=next(self._cl)
                 )
-                line_kw = self._kwargs.get("line_kw", dict())
-                self._fig.add_trace(go.Scatter(x=x, y=y, **merge({}, lkw, line_kw)))
+                kw = merge({}, lkw, s.rendering_kw)
+                self._fig.add_trace(go.Scatter(x=x, y=y, **kw))
 
             else:
                 raise NotImplementedError(
@@ -703,9 +575,17 @@ class PlotlyBackend(Plot):
                 )
 
     def _update_interactive(self, params):
+        np = import_module('numpy')
+        plotly = import_module(
+            'plotly',
+            import_kwargs={'fromlist': ['graph_objects', 'figure_factory']},
+            min_module_version='5.0.0')
+        create_quiver = plotly.figure_factory.create_quiver
+        merge = self.merge
+
         for i, s in enumerate(self.series):
             if s.is_interactive:
-                self.series[i].update_data(params)
+                self.series[i].params = params
                 if s.is_2Dline and s.is_parametric:
                     x, y, param = self.series[i].get_data()
                     self.fig.data[i]["x"] = x
@@ -715,7 +595,6 @@ class PlotlyBackend(Plot):
 
                 elif s.is_2Dline:
                     x, y = self.series[i].get_data()
-                    x, y, _ = self._detect_poles(x, y)
                     if s.is_geometry:
                         self.fig.data[i]["x"] = x
                     self.fig.data[i]["y"] = y
@@ -733,9 +612,6 @@ class PlotlyBackend(Plot):
                     self.fig.data[i]["y"] = y
                     self.fig.data[i]["z"] = z
 
-                # elif (s.is_3Dsurface and (not s.is_complex)) or (
-                #     s.is_3Dsurface and s.is_complex and (s.real or s.imag)
-                # ):
                 elif s.is_3Dsurface and (not s.is_domain_coloring):
                     x, y, z = self.series[i].get_data()
                     self.fig.data[i]["z"] = z
@@ -744,14 +620,6 @@ class PlotlyBackend(Plot):
                     _, _, zz = s.get_data()
                     self.fig.data[i]["z"] = zz
 
-                elif s.is_implicit:
-                    points = s.get_data()
-                    if len(points) == 2:
-                        raise NotImplementedError
-                    else:
-                        _, _, zz, ones, _ = points
-                        self.fig.data[i]["z"] = ones
-
                 elif s.is_vector and s.is_3D:
                     if s.is_streamlines:
                         raise NotImplementedError
@@ -763,43 +631,31 @@ class PlotlyBackend(Plot):
                 elif s.is_vector:
                     x, y, u, v = self.series[i].get_data()
                     if s.is_streamlines:
-                        streams = create_streamline(x[0, :], y[:, 0], u, v)
-                        data = streams.data[0]
-                        # TODO: iplot doesn't work with 2D streamlines. Why?
-                        # Is it possible that the sequential update of x and y
-                        # is the cause of the error? Since at every update,
-                        # len(x) = len(y) but those are different from before.
+                        # TODO: iplot doesn't work with 2D streamlines.
                         raise NotImplementedError
                     else:
-                        # default values
                         qkw = dict(
                             line_color=self.quivers_colors[i], scale=0.075, name=s.label
                         )
-                        # user-provided values
-                        quiver_kw = self._kwargs.get("quiver_kw", dict())
-                        quivers = create_quiver(x, y, u, v, **merge({}, qkw, quiver_kw))
+                        kw = merge({}, qkw, s.rendering_kw)
+                        quivers = create_quiver(x, y, u, v, **kw)
                         data = quivers.data[0]
                     self.fig.data[i]["x"] = data["x"]
                     self.fig.data[i]["y"] = data["y"]
 
                 elif s.is_complex:
                     if not s.is_3Dsurface:
-                        raise NotImplementedError
                         # TODO: for some unkown reason, domain_coloring and
                         # interactive plot don't like each other...
-                        # x, y, z, magn_angle, img, discr, colors = self._get_image(s)
-                        # self.fig.data[i]["z"] = img
-                        # self.fig.data[i]["x0"] = -4
-                        # self.fig.data[i]["y0"] = -5
-                        # # self.fig.data[i]["customdata"] = magn_angle
+                        raise NotImplementedError
                     else:
                         xx, yy, mag, angle, colors, colorscale = s.get_data()
                         self.fig.data[i]["z"] = mag
                         self.fig.data[i]["surfacecolor"] = angle
                         self.fig.data[i]["customdata"] = angle
                         m, M = min(angle.flatten()), max(angle.flatten())
-                        # show pi symbols on the colorbar if the range is close
-                        # enough to [-pi, pi]
+                        # show pi symbols on the colorbar if the range is
+                        # close enough to [-pi, pi]
                         if (abs(m + np.pi) < 1e-02) and (abs(M - np.pi) < 1e-02):
                             self.fig.data[i]["colorbar"]["tickvals"] = [
                                 m,
@@ -823,7 +679,7 @@ class PlotlyBackend(Plot):
 
     def _update_layout(self):
         self._fig.update_layout(
-            template=self._kwargs.get("theme", cfg["plotly"]["theme"]),
+            template=self._theme,
             width=None if not self.size else self.size[0],
             height=None if not self.size else self.size[1],
             title=r"<b>%s</b>" % ("" if not self.title else self.title),
@@ -834,7 +690,6 @@ class PlotlyBackend(Plot):
                 type=self.xscale,
                 showgrid=self.grid,  # thin lines in the background
                 zeroline=self.grid,  # thick line at x=0
-                visible=self.grid,  # numbers below
                 constrain="domain",
             ),
             yaxis=dict(
@@ -843,7 +698,6 @@ class PlotlyBackend(Plot):
                 type=self.yscale,
                 showgrid=self.grid,  # thin lines in the background
                 zeroline=self.grid,  # thick line at x=0
-                visible=self.grid,  # numbers below,
                 scaleanchor="x" if self.aspect == "equal" else None,
             ),
             margin=dict(
@@ -904,8 +758,9 @@ class PlotlyBackend(Plot):
         .. [#fn12] https://plotly.com/python/interactive-html-export/
 
         """
-        self._process_series(self._series)
-        self._update_layout()
+        if (len(self.series) > 0) and (len(self.fig.data) == 0):
+            self._process_series(self._series)
+            self._update_layout()
 
         ext = os.path.splitext(path)[1]
         if ext.lower() in [".html", ".html"]:
