@@ -1,8 +1,5 @@
-import numpy as np
+from sympy.external import import_module
 from PIL import ImageColor
-from matplotlib.colors import Colormap
-import plotly.colors
-from _plotly_utils.basevalidators import ColorscaleValidator
 
 
 def convert_colormap(cm, to, n=256):
@@ -30,6 +27,14 @@ def convert_colormap(cm, to, n=256):
         A new colormap. Note that the conversion is not guardanteed.
         The function returns the provided colormap if it cannot be converted.
     """
+    np = import_module('numpy')
+    matplotlib = import_module(
+        'matplotlib',
+        import_kwargs={'fromlist': ['colors']},
+        min_module_version='1.1.0',
+        catch=(RuntimeError,))
+    Colormap = matplotlib.colors.Colormap
+
     assert isinstance(to, str)
     to = to.lower()
     assert to in ["matplotlib", "plotly", "k3d", "bokeh"]
@@ -211,6 +216,11 @@ def _get_continuous_color(colorscale, intermed):
             An RGB color string in which the components are float numbers in the
             range [0, 255].
     """
+    plotly = import_module(
+        'plotly',
+        import_kwargs={'fromlist': ['colors']},
+        min_module_version='5.0.0')
+
     if len(colorscale) < 1:
         raise ValueError("colorscale must have at least one color")
 
@@ -261,9 +271,13 @@ def get_plotly_colors(colorscale_name, loc):
     =======
         An RGB list with components in the range [0, 1] or a list of RGB lists.
     """
+    _plotly_utils = import_module(
+        '_plotly_utils',
+        import_kwargs={'fromlist': ['basevalidators']})
+
     # first parameter: Name of the property being validated
     # second parameter: a string, doesn't really matter for our use cae
-    cv = ColorscaleValidator("colorscale", "")
+    cv = _plotly_utils.basevalidators.ColorscaleValidator("colorscale", "")
     # colorscale will be a list of lists: [[loc1, "rgb1"], [loc2, "rgb2"], ...]
     colorscale = cv.validate_coerce(colorscale_name)
 
@@ -275,13 +289,6 @@ def get_plotly_colors(colorscale_name, loc):
     return [float(t) / 255 for t in str_color[4:-1].split(",")]
 
 
-
-from vtk import (
-    VTK_FLOAT, vtkPoints, vtkStructuredGrid, vtkStreamTracer,
-     vtkPolyData, vtkPointSource, vtkRungeKutta4, vtkIdList
-)
-from vtk.util import numpy_support
-
 def get_seeds_points_entry_vector(xx, yy, zz, uu, vv, ww):
     """Returns an optimal list of seeds points to be used to generate 3D
     streamlines.
@@ -299,8 +306,7 @@ def get_seeds_points_entry_vector(xx, yy, zz, uu, vv, ww):
         points : np.ndarray
             [n x 3] matrix of seed-points coordinates.
     """
-
-    # TODO: OPTIMIZE THIS CODE PLEASE!!!!!!!!!
+    np = import_module('numpy')
 
     coords = np.stack([xx, yy, zz])
     # vector field
@@ -352,6 +358,7 @@ def get_seeds_points_entry_vector(xx, yy, zz, uu, vv, ww):
 
     return points
 
+
 def get_seeds_points(xx, yy, zz, uu, vv, ww, to_numpy=True, **kw):
     """
     Parameters
@@ -380,6 +387,9 @@ def get_seeds_points(xx, yy, zz, uu, vv, ww, to_numpy=True, **kw):
         - ``False``: a vtk object representing the seeds points.
 
     """
+    np = import_module('numpy')
+    import vtk
+    from vtk.util import numpy_support
     starts = kw.get("starts", None)
 
     if starts is None:
@@ -388,8 +398,8 @@ def get_seeds_points(xx, yy, zz, uu, vv, ww, to_numpy=True, **kw):
         if to_numpy:
             return points
 
-        seeds = vtkPolyData()
-        vtk_points = vtkPoints()
+        seeds = vtk.vtkPolyData()
+        vtk_points = vtk.vtkPoints()
         for p in points:
             vtk_points.InsertNextPoint(p)
         seeds.SetPoints(vtk_points)
@@ -410,8 +420,8 @@ def get_seeds_points(xx, yy, zz, uu, vv, ww, to_numpy=True, **kw):
         if to_numpy:
             return points
 
-        seeds = vtkPolyData()
-        seeds_points = vtkPoints()
+        seeds = vtk.vtkPolyData()
+        seeds_points = vtk.vtkPoints()
         for p in points:
             seeds_points.InsertNextPoint(p)
         seeds.SetPoints(seeds_points)
@@ -432,7 +442,7 @@ def get_seeds_points(xx, yy, zz, uu, vv, ww, to_numpy=True, **kw):
             )
             center = (xmax - xmin) / 2, (ymax - ymin) / 2, (zmax - zmin) / 2
 
-        seeds = vtkPointSource()
+        seeds = vtk.vtkPointSource()
         seeds.SetRadius(radius)
         seeds.SetCenter(*center)
         seeds.SetNumberOfPoints(npoints)
@@ -493,37 +503,40 @@ def compute_streamtubes(xx, yy, zz, uu, vv, ww, kwargs):
     extend the one provided by Plotly's Streamtube class. Read ``plot_vector``
     docstring for more information.
     """
+    np = import_module('numpy')
+    import vtk
+    from vtk.util import numpy_support
     n2, n1, n3 = xx.shape
 
     vector_field = np.array([uu.flatten(), vv.flatten(), ww.flatten()]).T
     vtk_vector_field = numpy_support.numpy_to_vtk(
-        num_array=vector_field, deep=True, array_type=VTK_FLOAT)
+        num_array=vector_field, deep=True, array_type=vtk.VTK_FLOAT)
     vtk_vector_field.SetName("vector_field")
 
-    points = vtkPoints()
+    points = vtk.vtkPoints()
     points.SetNumberOfPoints(n2 * n1 * n3)
     for i, (x, y, z) in enumerate(
         zip(xx.flatten(), yy.flatten(), zz.flatten())
     ):
         points.SetPoint(i, [x, y, z])
 
-    grid = vtkStructuredGrid()
+    grid = vtk.vtkStructuredGrid()
     grid.SetDimensions([n2, n1, n3])
     grid.SetPoints(points)
     grid.GetPointData().SetVectors(vtk_vector_field)
 
-    # copy the extracted dictionary: if multiple vector fields are being plotted
+    # copy the dictionary: if multiple vector fields are being plotted
     # simultaneously, we need the original again.
-    stream_kw = kwargs.get("stream_kw", dict()).copy()
-    starts = stream_kw.get("starts", None)
-    max_prop = stream_kw.pop("max_prop", 5000)
+    kwargs = kwargs.copy()
+    starts = kwargs.get("starts", None)
+    max_prop = kwargs.pop("max_prop", 5000)
 
-    streamer = vtkStreamTracer()
+    streamer = vtk.vtkStreamTracer()
     streamer.SetInputData(grid)
     streamer.SetMaximumPropagation(max_prop)
 
     seeds = get_seeds_points(xx, yy, zz, uu, vv, ww,
-        to_numpy=False, **stream_kw)
+        to_numpy=False, **kwargs)
 
     if starts is None:
         streamer.SetSourceData(seeds)
@@ -536,7 +549,7 @@ def compute_streamtubes(xx, yy, zz, uu, vv, ww, kwargs):
         streamer.SetIntegrationDirectionToBoth()
 
     streamer.SetComputeVorticity(0)
-    streamer.SetIntegrator(vtkRungeKutta4())
+    streamer.SetIntegrator(vtk.vtkRungeKutta4())
     streamer.Update()
 
     streamline = streamer.GetOutput()
@@ -548,7 +561,7 @@ def compute_streamtubes(xx, yy, zz, uu, vv, ww, kwargs):
 
     vtkLines = streamline.GetLines()
     vtkLines.InitTraversal()
-    point_list = vtkIdList()
+    point_list = vtk.vtkIdList()
 
     # extract vtk data to lists
     lines = []
