@@ -5,20 +5,16 @@ from spb.series import (
     ContourSeries,
     SliceVector3DSeries,
     _set_discretization_points,
-    InteractiveSeries,
+    InteractiveSeries
 )
 from spb.utils import _plot_sympify, _unpack_args, _split_vector, _is_range
-from sympy import S, sqrt, Expr, Tuple
+from sympy.core.containers import Tuple
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.core.expr import Expr
+from sympy.core.singleton import S
 from sympy.geometry import Plane
 
 
-"""
-TODO:
-*   check length of ranges and if the scalar free symbols are compatible with
-    the ones provided in the vector.
-"""
-
-
 def _build_series(*args, interactive=False, **kwargs):
     """Loop over args and create all the necessary series to display a vector
     plot.
@@ -38,8 +34,8 @@ def _build_series(*args, interactive=False, **kwargs):
     # add a scalar series only on 2D plots
     if all([s.is_2Dvector for s in series]):
         # don't pop this keyword: some backend needs it to decide the color
-        # for quivers (solid color if a scalar field is present, gradient color
-        # otherwise)
+        # for quivers (solid color if a scalar field is present, gradient
+        # color otherwise)
         scalar = kwargs.get("scalar", True)
         if (len(series) == 1) and (scalar is True):
             scalar_field = sqrt(split_expr[0] ** 2 + split_expr[1] ** 2)
@@ -56,7 +52,7 @@ def _build_series(*args, interactive=False, **kwargs):
             scalar_field = None
         else:
             raise ValueError(
-                "``scalar`` must be either:\n"
+                "`scalar` must be either:\n"
                 + "1. True, in which case the magnitude of the vector field "
                 + "will be plotted.\n"
                 + "2. a symbolic expression representing a scalar field.\n"
@@ -65,16 +61,6 @@ def _build_series(*args, interactive=False, **kwargs):
             )
 
         if scalar_field:
-            # TODO: does it makes sense to cross-check the free symbols of the
-            # scalar field with those of the vectors?
-
-            # if not fs.issuperset(scalar_field.free_symbols):
-            #     raise ValueError("The free symbols of the scalar field must be " +
-            #         "a subset of the free symbols in the vector. Received:\n"
-            #         "Vector free symbols: {}\n".format(fs) +
-            #         "Scalar field free symbols: {}".format(scalar_field.free_symbols)
-            #     )
-
             # plot the scalar field over the entire region covered by all
             # vector fields
             _minx, _maxx = float("inf"), -float("inf")
@@ -117,7 +103,8 @@ def _series(expr, *ranges, label="", interactive=False, **kwargs):
 
     # free symbols contained in the provided vector
     fs = set().union(*[e.free_symbols for e in split_expr])
-    # if we are building a parametric-interactive series, remove the parameters
+    # if we are building a parametric-interactive series, remove the
+    # parameters
     fs = fs.difference(params.keys())
 
     if split_expr[2] is S.Zero:  # 2D case
@@ -266,22 +253,15 @@ def _preprocess(*args, matrices=False, fill_ranges=True):
 def plot_vector(*args, show=True, **kwargs):
     """
     Plot a 2D or 3D vector field. By default, the aspect ratio of the plot
-    is set to ``aspect="equal"``.
+    is set to `aspect="equal"`.
 
     Typical usage examples are in the followings:
 
     - Plotting a vector field with a single range.
-
-      .. code-block::
-
-         plot(expr, range1, range2, range3 [optional], **kwargs)
+        `plot(expr, range1, range2, range3 [optional], **kwargs)`
 
     - Plotting multiple vector fields with different ranges and custom labels.
-
-      .. code-block::
-
-         plot((expr1, range1, range2, range3 [optional], label1),
-            (expr2, range4, range5, range6 [optional], label2), **kwargs)
+        `plot((expr1, range1, range2, range3 [optional], label1), (expr2, range4, range5, range6 [optional], label2), **kwargs)`
 
     Parameters
     ==========
@@ -304,7 +284,16 @@ def plot_vector(*args, show=True, **kwargs):
             If none is provided, the string representation of the vector will
             be used.
 
-    contours_kw : dict
+    aspect : (float, float) or str, optional
+        Set the aspect ratio of the plot. The value depends on the backend
+        being used. Read that backend's documentation to find out the
+        possible values.
+
+    backend : Plot, optional
+        A subclass of `Plot`, which will perform the rendering.
+        Default to `MatplotlibBackend`.
+
+    contour_kw : dict
         A dictionary of keywords/values which is passed to the backend
         contour function to customize the appearance. Refer to the plotting
         library (backend) manual for more informations.
@@ -315,7 +304,7 @@ def plot_vector(*args, show=True, **kwargs):
 
     n : int
         Set the same number of discretization points in all directions for
-        the quivers or streamlines. It overrides ``n1``, ``n2``, ``n3``.
+        the quivers or streamlines. It overrides `n1`, `n2`, `n3`.
         Default to 25.
 
     nc : int
@@ -331,20 +320,20 @@ def plot_vector(*args, show=True, **kwargs):
         Represents the scalar field to be plotted in the background of a 2D
         vector field plot. Can be:
 
-        - ``True``: plot the magnitude of the vector field. Only works when a
+        - `True`: plot the magnitude of the vector field. Only works when a
           single vector field is plotted.
-        - ``False``/``None``: do not plot any scalar field.
-        - ``Expr``: a symbolic expression representing the scalar field.
-        - ``list``/``tuple``: [scalar_expr, label], where the label will be
+        - `False`/`None`: do not plot any scalar field.
+        - `Expr`: a symbolic expression representing the scalar field.
+        - `list`/`tuple`: [scalar_expr, label], where the label will be
           shown on the colorbar.
 
         Default to True.
 
     show : boolean
-        The default value is set to ``True``. Set show to ``False`` and
+        The default value is set to `True`. Set show to `False` and
         the function will not display the plot. The returned instance of
-        the ``Plot`` class can then be used to save or display the plot
-        by calling the ``save()`` and ``show()`` methods respectively.
+        the `Plot` class can then be used to save or display the plot
+        by calling the `save()` and `show()` methods respectively.
 
     slice : Plane, list, Expr
         Plot the 3D vector field over the provided slice. It can be:
@@ -353,12 +342,12 @@ def plot_vector(*args, show=True, **kwargs):
         - a list of planes.
         - a symbolic expression representing a surface of two variables.
 
-        The number of discretization points will be ``n1``, ``n2``, ``n3``.
+        The number of discretization points will be `n1`, `n2`, `n3`.
         Note that:
 
         - only quivers plots are supported with slices. Streamlines plots
           are unaffected.
-        - ``n3`` will only be used with planes parallel to xz or yz.
+        - `n3` will only be used with planes parallel to xz or yz.
 
     streamlines : boolean
         Whether to plot the vector field using streamlines (True) or quivers
@@ -372,72 +361,73 @@ def plot_vector(*args, show=True, **kwargs):
         For 3D vector fields, by default the streamlines will start at the
         boundaries of the domain where the vectors are pointed inward.
         Depending on the vector field, this may results in too tight
-        streamlines. Use the ``starts`` keyword argument to control the
+        streamlines. Use the `starts` keyword argument to control the
         generation of streamlines:
 
-        - ``starts=None``: the default aforementioned behaviour.
-        - ``starts=dict(x=x_list, y=y_list, z=z_list)``: specify the starting
+        - `starts=None`: the default aforementioned behaviour.
+        - `starts=dict(x=x_list, y=y_list, z=z_list)`: specify the starting
           points of the streamlines.
-        - ``starts=True``: randomly create starting points inside the domain.
-          In this setup we can set the number of starting point with ``npoints``
+        - `starts=True`: randomly create starting points inside the domain.
+          In this setup we can set the number of starting point with `npoints`
           (default value to 200).
 
         If 3D streamlines appears to be cut short inside the specified domain,
-        try to increase ``max_prop`` (default value to 5000).
+        try to increase `max_prop` (default value to 5000).
 
 
     Examples
     ========
 
     .. plot::
-        :context: reset
-        :format: doctest
-        :include-source: True
+       :context: reset
+       :format: doctest
+       :include-source: True
 
-        >>> from sympy import symbols, sin, cos, Plane, Matrix, sqrt
-        >>> from spb.vectors import plot_vector
-        >>> x, y, z = symbols('x, y, z')
+       >>> from sympy import symbols, sin, cos, Plane, Matrix, sqrt
+       >>> from spb.vectors import plot_vector
+       >>> x, y, z = symbols('x, y, z')
 
     Quivers plot of a 2D vector field with a contour plot in background
     representing the vector's magnitude (a scalar field).
 
     .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
+       :context: close-figs
+       :format: doctest
+       :include-source: True
 
-        >>> plot_vector([-sin(y), cos(x)], (x, -3, 3), (y, -3, 3))
-        Plot object containing:
-        [0]: contour: sqrt(sin(y)**2 + cos(x)**2) for x over (-3.0, 3.0) and y over (-3.0, 3.0)
-        [1]: 2D vector series: [-sin(y), cos(x)] over (x, -3.0, 3.0), (y, -3.0, 3.0)
+       >>> plot_vector([-sin(y), cos(x)], (x, -3, 3), (y, -3, 3),
+       ...     quiver_kw=dict(color="black"), grid=False)
+       Plot object containing:
+       [0]: contour: sqrt(sin(y)**2 + cos(x)**2) for x over (-3.0, 3.0) and y over (-3.0, 3.0)
+       [1]: 2D vector series: [-sin(y), cos(x)] over (x, -3.0, 3.0), (y, -3.0, 3.0)
 
     Streamlines plot of a 2D vector field with no background scalar field.
 
     .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
+       :context: close-figs
+       :format: doctest
+       :include-source: True
 
-        >>> plot_vector([-sin(y), cos(x)], (x, -3, 3), (y, -3, 3),
-        ...     streamlines=True, scalar=None)
-        Plot object containing:
-        [0]: 2D vector series: [-sin(y), cos(x)] over (x, -3.0, 3.0), (y, -3.0, 3.0)
+       >>> plot_vector([-sin(y), cos(x)], (x, -3, 3), (y, -3, 3),
+       ...     streamlines=True, scalar=None)
+       Plot object containing:
+       [0]: 2D vector series: [-sin(y), cos(x)] over (x, -3.0, 3.0), (y, -3.0, 3.0)
 
 
     Plot multiple 2D vectors fields, setting a background scalar field to be
     the magnitude of the first vector.
 
     .. plot::
-        :context: close-figs
-        :format: doctest
-        :include-source: True
+       :context: close-figs
+       :format: doctest
+       :include-source: True
 
-        >>> plot_vector([-sin(y), cos(x)], [y, x], (x, -5, 5), (y, -3, 3), n=20,
-        ...     scalar=sqrt((-sin(y))**2 + cos(x)**2), legend=True)
-        Plot object containing:
-        [0]: contour: sqrt(sin(y)**2 + cos(x)**2) for x over (-5.0, 5.0) and y over (-3.0, 3.0)
-        [1]: 2D vector series: [-sin(y), cos(x)] over (x, -5.0, 5.0), (y, -3.0, 3.0)
-        [2]: 2D vector series: [y, x] over (x, -5.0, 5.0), (y, -3.0, 3.0)
+       >>> plot_vector([-sin(y), cos(x)], [y, x], (x, -5, 5), (y, -3, 3), n=20,
+       ...     scalar=sqrt((-sin(y))**2 + cos(x)**2), legend=True, grid=False)
+       Plot object containing:
+       [0]: contour: sqrt(sin(y)**2 + cos(x)**2) for x over (-5.0, 5.0) and y over (-3.0, 3.0)
+       [1]: 2D vector series: [-sin(y), cos(x)] over (x, -5.0, 5.0), (y, -3.0, 3.0)
+       [2]: 2D vector series: [y, x] over (x, -5.0, 5.0), (y, -3.0, 3.0)
 
     3D vector field.
 
